#ifndef DATASET_H
#define DATASET_H

#include "log4cxx/logger.h"
#include "Node.h"

#include "BSplineSet.h"

/**
  * Data set holding the tree and the B-Splines
  */
class DataSet
{
	static log4cxx::LoggerPtr logger;

public:
	DataSet(Node *_root, Node **_nodes, int _hierarchyElements);
	~DataSet();

	Node* getNode(int index);
	Node** getNodes();

	void initRelationArrays(int _relationCount);
	void addRelation(int index, int originIndex, int destinationIndex);
	Node* getRoot();

	void setHierarchyDepth(int _hierarchyDepth);
	void setMaxFanOut();

	void renderRooted();
	void renderRadial();
	void renderBalloon();

	void setBundlingStrength(float bundlingStrength);
	void setRelationVisability(bool visable);
	void setLcaRemoval(bool remove);
	void useTransparency(bool useTransparency);
	void init();

	float *getRootedTreeBoundaries();
	float *getRadialTreeBoundaries();
	float *getBalloonTreeBoundaries();

	void buildRootedTree();
	void buildRadialTree();
	void buildBalloonTree();

private:
	/** Boolean value indicating whether the Rooted Tree is built or not */
	bool rootedTreeBuilt;
	/** Boolean value indicating whether the Radial Tree is built or not */
	bool radialTreeBuilt;
	/** Boolean value indicating whether the Balloon Tree is built or not */
	bool balloonTreeBuilt;
	/** Boolean value indicating whether the relations (B-Splines) are rendered or not */
	bool showRelations;
	/** Pointer to the Node-object which is the root of the tree */
	Node *root;
	/** Array of pointers to the nodes of the tree */
	Node **nodes;
	/** Pointer to the BSpline set */
	BSplineSet *bSplineSet;
	/** Number of nodes in the tree */
	int hierarchyElements;
	/** Number of relations (B-Splines) in the tree */
	int relationCount;
	/** Index to insert a new relation into the corresponding arrays */
	int relationInsertionIndex;
	/** Number of layers of the tree */
	int hierarchyDepth;
	/** Maximum of childs of every node */
	int maxFanOut;
	/** Array with the IDs of nodes which are the origin of directed relationships to other nodes */
	int *originNodeIndices;
	/** Array with the IDs of nodes which are the destination of directed relationships to other nodes */
	int *destinationNodeIndices;
	/** Array with the boundaries of the Rooted Tree in the form xMin, yMin, xMax, yMax */
	float *rootedTreeBoundaries;
	/** Array with the boundaries of the Radial Tree in the form xMin, yMin, xMax, yMax */
	float *radialTreeBoundaries;
	/** Array with the boundaries of the Balloon Tree in the form xMin, yMin, xMax, yMax */
	float *balloonTreeBoundaries;
	/** ID of the Displaylist to draw the Rooted Tree */
	GLuint rootedTreeDisplayListID;
	/** ID of the Displaylist to draw the Radial Tree */
	GLuint radialTreeDisplayListID;
	/** ID of the Displaylist to draw the Balloon Tree */
	GLuint balloonTreeDisplayListID;
	/** Count of Nodes per level of the tree */
	unsigned int* nodesPerLevel;

	void findControlPolygons();
	void setTreeBoundaries();
};

#endif