#ifndef NODE_H
#define NODE_H

#include <vector>
#include <stdlib.h>

#include <QtOpenGL>
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library

#include "log4cxx/logger.h"

const float NODE_DIAMETER = 0.25f;
const float BALLOON_RADIUS_CONST = 0.5f; // 0.1f
const float BALLOON_NODE_CIRCLE_OFFSET = 0.2f;
const bool BALLOON_RENDER_HIERARCHYNODES = true;
const bool BALLOON_RENDER_HIERARCHYLINES = true;
const bool BALLOON_RENDER_CIRCLES = true;
const float RADIAL_RADIUS_CONST = 1.0f;
const float RADIAL_FREESPACE_ANGLE = 0.06f;

/**
  * Node of the Tree
  */
class Node
{
	static log4cxx::LoggerPtr logger;

public:
	Node(int _id);
	~Node();

	void setParent(Node *_parent);
	Node* getParent();

	void addChild(Node *_child);

	void setLevelsForSubTree(int actLevel);
	void setBoundariesOfSubTree(float *rootedBoundaries, float *radialBoundaries, float *balloonBoundaries);
	int getLevel();

	void renderRootedSubTree(float rootX, float rootY, int maxFanOut, int hierarchyDepth);
	void renderRadialSubTree(float rootX, float rootY, float rootAngle, int maxFanOut, int hierarchyDepth, unsigned int* nodesPerLevel);
	float renderBalloonSubTree(float rootX, float rootY, float rootAngle, float radius, int maxFanOut, int hierarchyDepth);
	int getMaximumFanOutOfSubtree();
	unsigned int getMaxNodesPerLevelOfSubtree(int hierarchyDepth);
	unsigned int* getNodesPerLevelOfSubtree(int hierarchyDepth);
	void getNodesPerLevel(unsigned int* nodesPerLevel);
	int getChildCount();
	unsigned int getMaxDepthOfSubtree();
	unsigned int getMinDepthOfSubtree();
	void getDepths(std::vector<unsigned int>* depths, unsigned int currentDepth);
	unsigned int getNodeCountOfSubtree();
	unsigned int getNodeCount(unsigned int nodeCount);
	void preComputeRadialSubTree(float rootAngle, unsigned int stopIndex, Node *childFound, bool down, unsigned int* nodesPerLevel);
	void computeSiblingsOfSubtree(unsigned int* nodesPerLevel);
	Node* getLeftSibling(Node* node, Node* lastNode, bool up);
	Node* getRightSibling(Node* node, Node* lastNode, bool up);
	Node* getLeftSiblingOfNode(unsigned int* nodesPerLevel);
	Node* getRightSiblingOfNode(unsigned int* nodesPerLevel);
	void computeSiblingIndex(unsigned int* nextLevelIndices);
	unsigned int getSiblingIndex();
	Node* getLeftSibling();
	Node* getRightSibling();

	int getID();

	float getRootedTreeX();
	float getRootedTreeY();

	float getRadialTreeX();
	float getRadialTreeY();

	float getBalloonTreeX();
	float getBalloonTreeY();

	// radial tree
	float getRadialLeftLimitAngle();
	float getRadialRightLimitAngle();
	float getRadialLeftBisectorLimitAngle();
	float getRadialRightBisectorLimitAngle();
	float getRadialAnglePerChild();
	float getRadialAngle();
	void computeLeftRadialBisectorLimitAngle();
	void computeRightRadialBisectorLimitAngle();
	void setRadialLeftBisectorLimitAngle(float angle);
	void setRadialRightBisectorLimitAngle(float angle);
	void setRadialAngle(float angle);
	void computeRadialBisectorLimitAngles();
	bool getIsLeftestChild();
	bool getIsRightestChild();
	void setIsLeftestChild(bool value);
	void setIsRightestChild(bool value);

	void setPreRootedX(float value);
	void setPreRootedY(float value);
	float getPreRootedX();
	float getPreRootedY();
	float getPreRootedXTmp();
	void preComputeRootedSubTree(bool up, unsigned int stopIndex);
	Node* findNodeAtMaxDepth(Node *node, int *maxDepth);

	// Balloon tree
	float preComputeBalloonTree(float parentAngle, unsigned int childCountOfParentNode);
	void setAngleOfNode(float angle);
	float getBalloonRadiusOfNode();
	float getBalloonAngleOfNode();

	void setNodeVisability(bool visable);
	void setLineVisability(bool visable);
	void setCircleVisability(bool visable);

private:
	/** Pointer to the parent node */
	Node *parent;
	/** Pointer to the left sibling node */
	Node *leftSibling;
	/** Pointer to the right sibling node */
	Node *rightSibling;
	/** Pointer to a vector with pointers to Node objects representing the childs of the node */
	std::vector<Node*> *childVector;
	/** Level of the node, the root has level 0 */
	int level;
	/** ID of the node */
	int id;
	/** X-Value of the node when the tree is rendered as Rooted Tree */
	float rootedTreeX;
	/** Y-Value of the node when the tree is rendered as Rooted Tree */
	float rootedTreeY;
	/** X-Value of the node when the tree is rendered as Radial Tree */
	float radialTreeX;
	/** Y-Value of the node when the tree is rendered as Radial Tree */
	float radialTreeY;
	/** X-Value of the node when the tree is rendered as Balloon Tree */
	float balloonTreeX;
	/** Y-Value of the node when the tree is rendered as Balloon Tree */
	float balloonTreeY;
	/** Boolean value indicating whether the nodes should be rendered or not */
	bool showNodes;
	/** Boolean value indicating whether the hierarchy lines should be rendered or not */
	bool showLines;
	/** Boolean value indicating whether the hierarchy circles should be rendered or not */
	bool showCircles;
	/** Radius of the node in the balloon tree */
	float balloonRadiusOfNode;
	/** Angle of the node in the balloon tree */
	float balloonAngleOfNode;
	/** Index of the nodes in one hierarchical level */
	unsigned int siblingIndex;

	// radial tree
	/** The left limit angle of the childs */
	float radialLeftLimitAngle;
	/** The right limit angle of the childs */
	float radialRightLimitAngle;
	/** The left bisector limit angle of the childs */
	float radialLeftBisectorLimitAngle;
	/** The right bisector limit angle of the childs */
	float radialRightBisectorLimitAngle;
	/** Angle for each child of that node */
	float radialAnglePerChild;
	/** Angle of that node in the radial tree */
	float radialAngle;
	/** Determines if the child is the leftest one */
	bool isLeftestChild;
	/** Determines if the child is the rightest one */
	bool isRightestChild;

	/** Precomputed X-Value of the node when the tree is rendered as Rooted Tree */
	float preRootedX;
	/** Precomputed X-Value of the node when the tree is rendered as Rooted Tree. It is used for transactions. */
	float preRootedXTmp;
	/** Precomputed Y-Value of the node when the tree is rendered as Rooted Tree */
	float preRootedY;

	void drawCircle(float x, float y, float z, float radius);
	void drawNode(float x, float y, float z, float radius);
};

#endif