#include "BSpline.h"

#include <QtOpenGL>
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library

using namespace log4cxx;

LoggerPtr BSpline::logger(Logger::getLogger("BSpline"));

/**
  * Constructs a new BSpline object
  * @param _basisFunction Pointer to the Basis Function Set to calculate the B-Spline curve
  * @param _controlPolygonXValues Array with x-Values of the initial control points
  * @param _controlPolygonYValues Array with y-Values of the initial control points
  * @param _controlPolygonSize Number of control points
  */
BSpline::BSpline(BasisFunction *_basisFunction, float *_controlPolygonXValues,
		float *_controlPolygonYValues, int _controlPolygonSize)
{
	basisFunction = _basisFunction;

	initialControlPolygonXValues = _controlPolygonXValues;
	initialControlPolygonYValues = _controlPolygonYValues;

	controlPolygonSize = _controlPolygonSize;

	bundlingControlPolygonXValues = new float[controlPolygonSize];
	bundlingControlPolygonYValues = new float[controlPolygonSize];

	for (int i = 0; i < controlPolygonSize; i++)
	{
		bundlingControlPolygonXValues[i] = initialControlPolygonXValues[i];
		bundlingControlPolygonYValues[i] = initialControlPolygonYValues[i];
	}

	splinePoints = basisFunction->getNumberOfSplinePoints();

	xValues = new float[splinePoints];
	yValues = new float[splinePoints];

	lengthSquared = 0.0f;

	init();
}

/**
  * Destroys the object
  */
BSpline::~BSpline()
{
	delete [] initialControlPolygonXValues;
	delete [] initialControlPolygonYValues;

	delete [] bundlingControlPolygonXValues;
	delete [] bundlingControlPolygonYValues;

	delete [] xValues;
	delete [] yValues;
}

/**
  * Renders the B-Spline curve
  * @param alpha The alpha value of the spline
  */
void BSpline::render(float alpha)
{
	float r = 0.0f;
	float g = 1.0f;
	float inc = 1.0f / static_cast<float>(splinePoints - 1);

	glBegin(GL_LINE_STRIP);

	for (int i = 0; i < splinePoints; i++)
	{	glColor4f(r, g, 0.0f, alpha);
		glVertex3f(xValues[i], yValues[i], 0.1f);

		r += inc;
		g -= inc;
	}

	glEnd();
}

/**
  * Initializes the B-Spline by filling the arrays with the x- and y-values of the underlying spline points
  */
void BSpline::init()
{
	float xValue;
	float yValue;

	for (int i = 0; i < splinePoints; i++)
	{
		xValue = 0.0f;
		yValue = 0.0f;

		for (int j = 0; j < controlPolygonSize; j++)
		{
			xValue += bundlingControlPolygonXValues[j] * basisFunction->getValue(j, i);
			yValue += bundlingControlPolygonYValues[j] * basisFunction->getValue(j, i);
		}

		xValues[i] = xValue;
		yValues[i] = yValue;
	}

	computeLength();
}

/**
  * Rebuilds the B-Spline when the bundling strength changes
  * @param beta the new bundling strength which modifies the initial control polygon
  */
void BSpline::rebuild(float beta)
{
	for (int i = 0; i < controlPolygonSize; i++)
	{
		bundlingControlPolygonXValues[i] = beta * initialControlPolygonXValues[i] +
			(1.0f - beta) * (initialControlPolygonXValues[0] +
			(static_cast<float>(i) / static_cast<float>(controlPolygonSize - 1)) * 
			(initialControlPolygonXValues[controlPolygonSize - 1] -
			initialControlPolygonXValues[0]));

		bundlingControlPolygonYValues[i] = beta * initialControlPolygonYValues[i] +
			(1.0f - beta) * (initialControlPolygonYValues[0] +
			(static_cast<float>(i) / static_cast<float>(controlPolygonSize - 1)) * 
			(initialControlPolygonYValues[controlPolygonSize - 1] -
			initialControlPolygonYValues[0]));
	}

	init();
}

/**
  * Computes the length of the B-Spline
  */
void BSpline::computeLength()
{	
	float xDif;
	float yDif;

	for (int i = 1; i < splinePoints; i++)
	{
		xDif = xValues[i] - xValues[i - 1];
		yDif = yValues[i] - yValues[i - 1];
		lengthSquared += xDif * xDif + yDif * yDif;
	}
}

/**
  * Returns the squared length of the B-Spline
  * @return the squared length of the B-Spline
  */
float BSpline::getSquaredLength()
{
	return lengthSquared;
}