#include "BSplineSet.h"

#include <common.h>

using namespace log4cxx;

LoggerPtr BSplineSet::logger(Logger::getLogger("BSplineSet"));

/**
  * Constructs a new BSplineSet object
  * @param _hierarchyDepth Number of layers of the underlying tree
  */
BSplineSet::BSplineSet(int _hierarchyDepth)
{
	pointsPerSpline = 50;
	bundlingStrength = 0.0f;
	hierarchyDepth = _hierarchyDepth;
	basisFunctionArray = new BasisFunction*[2 * hierarchyDepth - 1];

	for (int i = 0; i < 2 * hierarchyDepth - 1; i++)
	{
		basisFunctionArray[i] = NULL;
	}

	rootedBSplineVector = new std::vector<BSpline*>();
	radialBSplineVector = new std::vector<BSpline*>();
	balloonBSplineVector = new std::vector<BSpline*>();

	rootedBSplineWithLcaVector = new std::vector<BSpline*>();
	radialBSplineWithLcaVector = new std::vector<BSpline*>();
	balloonBSplineWithLcaVector = new std::vector<BSpline*>();

	removeLca = true;
	transparency = true;
}

/**
  * Destroys the object
  */
BSplineSet::~BSplineSet()
{
	rootedBSplineVector->clear();
	rootedBSplineVector->clear();
	radialBSplineVector->clear();
	balloonBSplineVector->clear();

	rootedBSplineWithLcaVector->clear();
	radialBSplineWithLcaVector->clear();
	balloonBSplineWithLcaVector->clear();

	delete rootedBSplineVector;
	delete radialBSplineVector;
	delete balloonBSplineVector;
	
	delete rootedBSplineWithLcaVector;
	delete radialBSplineWithLcaVector;
	delete balloonBSplineWithLcaVector;

	for (int i = 0; i < 2 * hierarchyDepth - 1; i++)
	{
		if (basisFunctionArray[i] != NULL)
		{
			delete basisFunctionArray[i];
		}
	}

	delete [] basisFunctionArray;
}

/**
  * Sets the value of the bundling strength which modifies the control polygons of the B-Splines
  * @param _bundlingStrength the new bundling strength
  */
void BSplineSet::setBundlingStrength(float _bundlingStrength)
{
	bundlingStrength = _bundlingStrength;
	
	std::vector<BSpline*>::iterator it;

	for (it = rootedBSplineVector->begin(); it != rootedBSplineVector->end(); it++)
	{
		(*it)->rebuild(bundlingStrength);
	}

	for (it = radialBSplineVector->begin(); it != radialBSplineVector->end(); it++)
	{
		(*it)->rebuild(bundlingStrength);
	}

	for (it = balloonBSplineVector->begin(); it != balloonBSplineVector->end(); it++)
	{
		(*it)->rebuild(bundlingStrength);
	}

	for (it = rootedBSplineWithLcaVector->begin(); it != rootedBSplineWithLcaVector->end(); it++)
	{
		(*it)->rebuild(bundlingStrength);
	}

	for (it = radialBSplineWithLcaVector->begin(); it != radialBSplineWithLcaVector->end(); it++)
	{
		(*it)->rebuild(bundlingStrength);
	}

	for (it = balloonBSplineWithLcaVector->begin(); it != balloonBSplineWithLcaVector->end(); it++)
	{
		(*it)->rebuild(bundlingStrength);
	}
}

/**
  * Adds a new B-Spline to the B-Spline set
  * @param controlPolygon array with pointers to Node-objects representing the control points
  * @param size number of control points inside the array
  * @param lcaIndex index of the least common ancestor inside the array
  */
void BSplineSet::addBSpline(Node **controlPolygon, int size, int lcaIndex)
{
	int index;

	if (size > 3)
	{
		// Remove LCA from the path
		if (basisFunctionArray[size - 2] == NULL)
		{
			basisFunctionArray[size - 2] = new BasisFunction(size - 1, pointsPerSpline);
		}

		//B-Spline for the Rooted Tree without LCA
		float *rootedControlPolygonX = new float[size - 1];
		float *rootedControlPolygonY = new float[size - 1];

		index = 0;
		for (int i = 0; i < size; i++)
		{	
			if (i == lcaIndex)
			{
				continue;
			}

			rootedControlPolygonX[index] = controlPolygon[i]->getRootedTreeX();
			rootedControlPolygonY[index] = controlPolygon[i]->getRootedTreeY();
			index++;
		}

		rootedBSplineVector->push_back(new BSpline(basisFunctionArray[size - 2],
			rootedControlPolygonX, rootedControlPolygonY, size - 1));
		
		//B-Spline for the Radial Tree without LCA
		float *radialControlPolygonX = new float[size - 1];
		float *radialControlPolygonY = new float[size - 1];

		index = 0;
		for (int i = 0; i < size; i++)
		{
			if (i == lcaIndex)
			{
				continue;
			}

			radialControlPolygonX[index] = controlPolygon[i]->getRadialTreeX();
			radialControlPolygonY[index] = controlPolygon[i]->getRadialTreeY();
			index++;
		}

		radialBSplineVector->push_back(new BSpline(basisFunctionArray[size - 2],
			radialControlPolygonX, radialControlPolygonY, size - 1));


		//B-Spline for the Balloon Tree without LCA
		float *balloonControlPolygonX = new float[size - 1];
		float *balloonControlPolygonY = new float[size - 1];

		index = 0;
		for (int i = 0; i < size; i++)
		{	
			if (i == lcaIndex)
			{
				continue;
			}

			balloonControlPolygonX[index] = controlPolygon[i]->getBalloonTreeX();
			balloonControlPolygonY[index] = controlPolygon[i]->getBalloonTreeY();
			index++;
		}

		balloonBSplineVector->push_back(new BSpline(basisFunctionArray[size - 2],
			balloonControlPolygonX, balloonControlPolygonY, size - 1));

	} else
	{
		// Don't remove the LCA from the path
		if (basisFunctionArray[size - 1] == NULL)
		{
			basisFunctionArray[size - 1] = new BasisFunction(size, pointsPerSpline);
		}

		//B-Spline for the Rooted Tree
		float *rootedControlPolygonX = new float[size];
		float *rootedControlPolygonY = new float[size];

		for (int i = 0; i < size; i++)
		{
			rootedControlPolygonX[i] = controlPolygon[i]->getRootedTreeX();
			rootedControlPolygonY[i] = controlPolygon[i]->getRootedTreeY();
		}

		rootedBSplineVector->push_back(new BSpline(basisFunctionArray[size - 1],
			rootedControlPolygonX, rootedControlPolygonY, size));

		//B-Spline for the Radial Tree
		float *radialControlPolygonX = new float[size];
		float *radialControlPolygonY = new float[size];

		for (int i = 0; i < size; i++)
		{
			radialControlPolygonX[i] = controlPolygon[i]->getRadialTreeX();
			radialControlPolygonY[i] = controlPolygon[i]->getRadialTreeY();
		}

		radialBSplineVector->push_back(new BSpline(basisFunctionArray[size - 1],
			radialControlPolygonX, radialControlPolygonY, size));

		//B-Spline for the Balloon Tree
		float *balloonControlPolygonX = new float[size];
		float *balloonControlPolygonY = new float[size];

		for (int i = 0; i < size; i++)
		{
			balloonControlPolygonX[i] = controlPolygon[i]->getBalloonTreeX();
			balloonControlPolygonY[i] = controlPolygon[i]->getBalloonTreeY();
		}

		balloonBSplineVector->push_back(new BSpline(basisFunctionArray[size - 1],
			balloonControlPolygonX, balloonControlPolygonY, size));
	}

	//B-Splines with LCA

	if (basisFunctionArray[size - 1] == NULL)
	{
		basisFunctionArray[size - 1] = new BasisFunction(size, pointsPerSpline);
	}

	//B-Spline for the Rooted Tree
	float *rootedControlPolygonWithLcaX = new float[size];
	float *rootedControlPolygonWithLcaY = new float[size];

	for (int i = 0; i < size; i++)
	{
		rootedControlPolygonWithLcaX[i] = controlPolygon[i]->getRootedTreeX();
		rootedControlPolygonWithLcaY[i] = controlPolygon[i]->getRootedTreeY();
	}

	rootedBSplineWithLcaVector->push_back(new BSpline(basisFunctionArray[size - 1],
		rootedControlPolygonWithLcaX, rootedControlPolygonWithLcaY, size));

	//B-Spline for the Radial Tree
	float *radialControlPolygonWithLcaX = new float[size];
	float *radialControlPolygonWithLcaY = new float[size];

	for (int i = 0; i < size; i++)
	{
		radialControlPolygonWithLcaX[i] = controlPolygon[i]->getRadialTreeX();
		radialControlPolygonWithLcaY[i] = controlPolygon[i]->getRadialTreeY();
	}

	radialBSplineWithLcaVector->push_back(new BSpline(basisFunctionArray[size - 1],
		radialControlPolygonWithLcaX, radialControlPolygonWithLcaY, size));

	//B-Spline for the Balloon Tree
	float *balloonControlPolygonWithLcaX = new float[size];
	float *balloonControlPolygonWithLcaY = new float[size];

	for (int i = 0; i < size; i++)
	{
		balloonControlPolygonWithLcaX[i] = controlPolygon[i]->getBalloonTreeX();
		balloonControlPolygonWithLcaY[i] = controlPolygon[i]->getBalloonTreeY();
	}

	balloonBSplineWithLcaVector->push_back(new BSpline(basisFunctionArray[size - 1],
		balloonControlPolygonWithLcaX, balloonControlPolygonWithLcaY, size));
}

/**
  * Renders the set of B-Splines for the Rooted Tree
  */
void BSplineSet::renderRooted()
{
	float factor;
	float dif;
	int lastIndex;

	std::vector<BSpline*>::iterator it;

	if (removeLca)
	{
		lastIndex = rootedBSplineVector->size() - 1;
		dif = (*rootedBSplineVector)[0]->getSquaredLength() - 
			(*rootedBSplineVector)[lastIndex]->getSquaredLength();

		for (it = rootedBSplineVector->begin(); it != rootedBSplineVector->end(); it++)
		{
			factor = ((*it)->getSquaredLength() - (*rootedBSplineVector)[lastIndex]->getSquaredLength()) / dif;
			
			if (transparency)
			{
				(*it)->render(pow(E, -factor));
			} else
			{
				(*it)->render(1.0f);
			}	
		}
	} else
	{
		lastIndex = rootedBSplineWithLcaVector->size() - 1;
		dif = (*rootedBSplineWithLcaVector)[0]->getSquaredLength() - 
			(*rootedBSplineWithLcaVector)[lastIndex]->getSquaredLength();
			
		for (it = rootedBSplineWithLcaVector->begin(); it != rootedBSplineWithLcaVector->end(); it++)
		{
			factor = ((*it)->getSquaredLength() - (*rootedBSplineWithLcaVector)[lastIndex]->getSquaredLength()) / dif;
			
			if (transparency)
			{
				(*it)->render(pow(E, -factor));
			} else
			{
				(*it)->render(1.0f);
			}
		}
	}
}

/**
  * Renders the set of B-Splines for the Radial Tree
  */
void BSplineSet::renderRadial()
{	
	float factor;
	float dif;
	int lastIndex;

	std::vector<BSpline*>::iterator it;

	if (removeLca)
	{
		lastIndex = radialBSplineVector->size() - 1;
		dif = (*radialBSplineVector)[0]->getSquaredLength() - 
			(*radialBSplineVector)[lastIndex]->getSquaredLength();

		for (it = radialBSplineVector->begin(); it != radialBSplineVector->end(); it++)
		{
			factor = ((*it)->getSquaredLength() - (*radialBSplineVector)[lastIndex]->getSquaredLength()) / dif;
			
			if (transparency)
			{
				(*it)->render(pow(E, -factor));
			} else
			{
				(*it)->render(1.0f);
			}
		}
	} else
	{
		lastIndex = radialBSplineWithLcaVector->size() - 1;
		dif = (*radialBSplineWithLcaVector)[0]->getSquaredLength() - 
			(*radialBSplineWithLcaVector)[lastIndex]->getSquaredLength();			

		for (it = radialBSplineWithLcaVector->begin(); it != radialBSplineWithLcaVector->end(); it++)
		{
			factor = ((*it)->getSquaredLength() - (*radialBSplineWithLcaVector)[lastIndex]->getSquaredLength()) / dif;
			
			if (transparency)
			{
				(*it)->render(pow(E, -factor));
			} else
			{
				(*it)->render(1.0f);
			}
		}
	}
}

/**
  * Renders the set of B-Splines for the Balloon Tree
  */
void BSplineSet::renderBalloon()
{
	float factor;
	float dif;
	int lastIndex;

	std::vector<BSpline*>::iterator it;

	if (removeLca)
	{	
		lastIndex = balloonBSplineVector->size() - 1;
		dif = (*balloonBSplineVector)[0]->getSquaredLength() - 
			(*balloonBSplineVector)[lastIndex]->getSquaredLength();			

		for (it = balloonBSplineVector->begin(); it != balloonBSplineVector->end(); it++)
		{
			factor = ((*it)->getSquaredLength() - (*balloonBSplineVector)[lastIndex]->getSquaredLength()) / dif;
			
			if (transparency)
			{
				(*it)->render(pow(E, -factor));
			} else
			{
				(*it)->render(1.0f);
			}
		}
	} else
	{
		lastIndex = balloonBSplineWithLcaVector->size() - 1;
		dif = (*balloonBSplineWithLcaVector)[0]->getSquaredLength() - 
			(*balloonBSplineWithLcaVector)[lastIndex]->getSquaredLength();

		for (it = balloonBSplineWithLcaVector->begin(); it != balloonBSplineWithLcaVector->end(); it++)
		{	
			factor = ((*it)->getSquaredLength() - (*balloonBSplineWithLcaVector)[lastIndex]->getSquaredLength()) / dif;
			
			if (transparency)
			{
				(*it)->render(pow(E, -factor));
			} else
			{
				(*it)->render(1.0f);
			}
		}
	}
}

/**
  * Sets a boolean value indicating whether the lca should be removed or not
  * @param _removeLca boolean value indicating whether the lca should be removed or not
  */
void BSplineSet::setLcaRemoval(bool _removeLca)
{
	removeLca = _removeLca; 
}

/**
  * Sets a boolean value indicating whether transparency is used for rendering the splines or not
  * @param _transparency boolean value indicating whether transparency is used for rendering the splines or not
  */
void BSplineSet::useTransparency(bool _transparency)
{
	transparency = _transparency;
}

/**
  * Sorts the B-Splines by length so that they are rendered from the longest to the shortest
  */
void BSplineSet::sortBSplinesByLength()
{
	quickSort(0, rootedBSplineVector->size() - 1, rootedBSplineVector);
	quickSort(0, radialBSplineVector->size() - 1, radialBSplineVector);
	quickSort(0, balloonBSplineVector->size() - 1, balloonBSplineVector);

	quickSort(0, rootedBSplineWithLcaVector->size() - 1, rootedBSplineWithLcaVector);
	quickSort(0, radialBSplineWithLcaVector->size() - 1, radialBSplineWithLcaVector);
	quickSort(0, balloonBSplineWithLcaVector->size() - 1, balloonBSplineWithLcaVector);
}

/**
  * Implementation of the QuickSort Algorithm from Tony Hoare to sort the B-Splines by length
  * The Splines are sorted from the longest to the shortest
  * @param left Left index, usually 0
  * @param right Right index, usually splineVector->size() - 1
  * @param splineVector Pointer to the splineVector with the pointers to the B-Spline objects
  */
void BSplineSet::quickSort(int left, int right, std::vector<BSpline*> *splineVector)
{	
	int pivot;

	if (left < right)
	{
		pivot = split(left, right, splineVector);
		quickSort(left, pivot - 1, splineVector);
		quickSort(pivot + 1, right, splineVector);
	}
}

/**
  * Finds a pivot element, brings the other elements to the correct side and returns its index
  * @param left Left index
  * @param right Right index
  * @param splineVector Pointer to the splineVector with the pointers to the B-Spline objects
  */
int BSplineSet::split(int left, int right, std::vector<BSpline*> *splineVector)
{	
	int i = left + 1;
	int j = right;
	float pivot = (*splineVector)[left]->getSquaredLength();
	BSpline* temp;

	do
	{
		while ((*splineVector)[i]->getSquaredLength() >= pivot && i < right)
		{
			i++;
		}

		while ((*splineVector)[j]->getSquaredLength() <= pivot && j > left)
		{
			j--;
		}

		if (i < j)
		{
			temp = (*splineVector)[i];
			(*splineVector)[i] = (*splineVector)[j];
			(*splineVector)[j] = temp;
		}

	} while (i < j);

	if ((*splineVector)[j]->getSquaredLength() > pivot)
	{
		temp = (*splineVector)[j];
		(*splineVector)[j] = (*splineVector)[left];
		(*splineVector)[left] = temp;
	}

	return j;
}