#include "BasisFunction.h"
#include "common.h"

using namespace log4cxx;

LoggerPtr BasisFunction::logger(Logger::getLogger("BasisFunction"));

/**
  * Constructs a new BasisFunction object
  * @param _numberOfControlPoints Number of control points for the B-Spline
  * @param _splinePoints Number of points in the line strip to draw the B-Spline
  */
BasisFunction::BasisFunction(int _numberOfControlPoints, int _splinePoints)
{	
	numberOfControlPoints = _numberOfControlPoints;
	splinePoints = _splinePoints;

	valueArray = new double*[numberOfControlPoints];

	for (int i = 0; i < numberOfControlPoints; i++)
	{
		valueArray[i] = new double[splinePoints];
	}

	switch (numberOfControlPoints)
	{
		case 2:
			p = 2;
			break;
		case 3:
			p = 3;
			break;
		default:
			p = 4;
			break;
	}

	n = numberOfControlPoints + p;

	initKnotVector();
	initValueArray();
}

/**
  * Destroys the object
  */
BasisFunction::~BasisFunction()
{
	delete [] knotVector;

	for (int i = 0; i < numberOfControlPoints; i++)
	{
		delete [] valueArray[i];
	}

	delete [] valueArray;
}

/**
  * Initializes the knot vector to be a uniform knot vector
  */
void BasisFunction::initKnotVector()
{

	knotVector = new int[n];
	int index = p;
	int rest;
	int knotValue;

	int endValue = n - 2 * p + 1;

	for (int i = 0; i < p; i++)
	{
		knotVector[i] = 0;
		knotVector[n - i - 1] = endValue;
	}

	if (n > 2 * p)
	{
		rest = n - 2 * p;
		knotValue = 1;
		index = p;

		for (int i = 0; i < rest; i++)
		{
			knotVector[index] = knotValue;
			knotValue++;
			index++;
		}
	}
}

/**
  * Returns the value of the i-th BasisFunction at the splinePoint-th Splinepoint
  * @param i Index of the BasisFunction starting at 0
  * @param splinePoint Index of the Splinepoint starting at 0
  * @return the value of the i-th BasisFunction at the splinePoint-th Splinepoint
  */
double BasisFunction::getValue(int i, int splinePoint)
{
	if (splinePoint < 0 || splinePoint >= splinePoints)
	{
		return 0.0;
	}

	return valueArray[i][splinePoint];
}

/**
  * Initializes the value array by filling it with the weights for all
  * control points at every spline point
  */
void BasisFunction::initValueArray()
{
	double u;

	double inc = knotVector[n - 1] - knotVector[0];
	inc /= (splinePoints - 1);

	for (int i = 0; i < numberOfControlPoints; i++)
	{	
		u = knotVector[0];

		for (int j = 0; j < splinePoints; j++)
		{
			if (i == numberOfControlPoints - 1 && j == splinePoints - 1)
			{
				valueArray[i][j] = 1.0;
			} else
			{
				valueArray[i][j] = coxRecursion(i + 1, p, u);
			}

			u += inc;
		}
	}
}

/**
  * Calculates the value of the recursion formula of Cox/De Boor
  * @param i Index of the function inside [1, n-p]
  * @param p degree parameter, equal to degree-1
  * @param u running parameter to define the place where the function should be evaluated, must be inside the range of the knot vector
  * @return the value of the recursion formula of Cox/De Boor
  */
double BasisFunction::coxRecursion(int i, int p, double u)
{
	if (p == 1)
	{
		if (u >= knotVector[i - 1] && u <= knotVector[i])
		{
			return 1.0;
		}

		return 0.0;
	}

	double z;
	double n;

	double koef1;
	double koef2;

	z = u - knotVector[i - 1];
	n = knotVector[i + p - 2] - knotVector[i - 1];

	if (n == 0.0)
	{
		koef1 = 0.0;
	} else
	{
		koef1 = z / n;
	}

	z = knotVector[i + p - 1] - u;
	n = knotVector[i + p - 1] - knotVector[i];

	if (n == 0.0)
	{
		koef2 = 0.0;
	} else
	{
		koef2 = z / n;
	}

	if (koef1 == 0.0 && koef2 == 0.0)
	{
		return 0.0;
	}

	if (koef1 == 0.0)
	{
		return (koef2 * coxRecursion(i + 1, p - 1, u));
	}

	if (koef2 == 0.0)
	{
		return (koef1 * coxRecursion(i, p - 1, u));
	}

	return (koef1 * coxRecursion(i, p - 1, u) +
		koef2 * coxRecursion(i + 1, p - 1, u));
}

/**
  * Returns the number of spline points to draw the B-Spline
  * @return the number of spline points to draw the B-Spline
  */
int BasisFunction::getNumberOfSplinePoints()
{
	return splinePoints;
}