#include <windows.h>
#include <fstream>
#include <iostream>
#include <string>
#include <math.h>

#include "Camera.h"
#include "common.h"

using namespace log4cxx;

LoggerPtr Camera::logger(Logger::getLogger("Camera"));

/**
  * Constructs a new camera object
  */
Camera::Camera() 
{
	translationX = 0.0f;
	translationY = 0.0f;
	oldTranslationX = 0.0f;
	oldTranslationY = 0.0f;
	zoom = 5.0f;
	oldZoom = 1.0f;
	rightButtonPressed = false;
	leftButtonPressed = false;
	width = 1.0f;
	height = 1.0f;
}

/**
  * Destroys the camera object
  */
Camera::~Camera() 
{

}

/**
  * Applies the camera transformation and sets the projection
  * @params aspectRatio Current aspect ratio of the window
  */
void Camera::apply(float aspectRatio) 
{
	if (aspectRatio >= 1.0f) {
		gluOrtho2D(-aspectRatio * zoom + translationX, aspectRatio * zoom + translationX, -zoom + translationY, zoom + translationY);
		width = 2.0f * aspectRatio * zoom;
		height = 2.0f * zoom;
	} else {
		gluOrtho2D(-zoom + translationX, zoom + translationX, -(zoom / aspectRatio) + translationY, (zoom / aspectRatio) + translationY);
		width = 2.0f * zoom;
		height = ((2.0f * zoom) / aspectRatio);
	}
}

/**
  * Processes a wheel event from the mouse
  * @param event the wheel event
  */
void Camera::processWheelEvent(QWheelEvent *event) 
{
	zoom += static_cast<float>(-event->delta()) / 1000.0f;

	if(zoom < 0.0f) zoom = 0.01f;
}

/**
  * Process the mouse event
  * @param button Button which is pressed or released
  * @param pressed Boolean value indicating whether the button is pressed or not
  * @param screenX X-Coordinate of the mouse on the screen
  * @param screenY Y-Coordinate of the mouse on the screen
  * @param vX X-Coordinate of the top-left corner of the viewport
  * @param vY Y-Coordinate of the top-left corner of the viewport
  * @param vWidth Width of the viewport
  * @param vHeight Height of the viewport
  */
void Camera::processMouseEvent(Qt::MouseButton button, bool pressed, int screenX, int screenY, int vX, int vY, int vWidth, int vHeight) {
	if(!leftButtonPressed && (button == Qt::RightButton) && pressed) {
		pressXRatio = static_cast<float>(screenX - vX) / static_cast<float>(vWidth - 1);
		pressYRatio = 1.0f - static_cast<float>(screenY - vY) / static_cast<float>(vHeight - 1);

		oldZoom = zoom;
		rightButtonPressed = true;
	}

	if(!rightButtonPressed && button == Qt::LeftButton && pressed) {
		pressXRatio = static_cast<float>(screenX - vX) / static_cast<float>(vWidth - 1);
		pressYRatio = 1.0f - static_cast<float>(screenY - vY) / static_cast<float>(vHeight - 1);
		
		oldTranslationX = translationX;
		oldTranslationY = translationY;

		leftButtonPressed = true;
	}

	if((button == Qt::RightButton) && (!pressed)) rightButtonPressed = false;
	if((button == Qt::LeftButton) && (!pressed)) leftButtonPressed = false;
}

/**
  * Process the motion event
  * @param screenX X-Coordinate of the mouse on the screen
  * @param screenY Y-Coordinate of the mouse on the screen
  * @param vX X-Coordinate of the top-left corner of the viewport
  * @param vY Y-Coordinate of the top-left corner of the viewport
  * @param vWidth Width of the viewport
  * @param vHeight Height of the viewport
  */
void Camera::processMotionEvent(int screenX, int screenY, int vX, int vY, int vWidth, int vHeight) {
	
	if((rightButtonPressed == false) && (leftButtonPressed == true)) {
		//Translate the camera
		float actPressXRatio = static_cast<float>(screenX - vX) / static_cast<float>(vWidth - 1);
		float actPressYRatio = 1.0f - static_cast<float>(screenY - vY) / static_cast<float>(vHeight - 1);

		translationX = oldTranslationX + (pressXRatio - actPressXRatio) * width;
		translationY = oldTranslationY + (pressYRatio - actPressYRatio) * height;
	} else if ((rightButtonPressed == true) && (leftButtonPressed == false)) {
		//Zoom with the camera
		float actPressXRatio = static_cast<float>(screenX - vX) / static_cast<float>(vWidth - 1);
		float actPressYRatio = 1.0f - static_cast<float>(screenY - vY) / static_cast<float>(vHeight - 1);
		zoom = oldZoom * (1.0f - (actPressYRatio - pressYRatio));
	}
}

/**
  * Resets the camera transformations
  */
void Camera::reset() 
{
	translationX = 0.0f;
	translationY = 0.0f;
	oldTranslationX = 0.0f;
	oldTranslationY = 0.0f;
	zoom = 5.0f;
	oldZoom = 1.0f;
	rightButtonPressed = false;
	leftButtonPressed = false;
	width = 1.0f;
	height = 1.0f;
}

/** 
  * Returns the camera translation in x direction
  * @return camera translation in x direction
  */
float Camera::getTranslationX() 
{
	return translationX;
}

/** 
  * Returns the camera translation in y direction
  * @return camera translation in y direction
  */
float Camera::getTranslationY() 
{
	return translationY;
}

/**
  * Returns the zoom of the camera
  * @return zoom of the camera
  */
float Camera::getZoom() 
{
	return zoom;
}

/**
  * Sets the translation in x direction of the camera
  * @params value translation in x direction of the camera
  */
void Camera::setTranslationX(float value) 
{
	translationX = value;
}

/**
  * Sets the translation in y direction of the camera
  * @params value translation in y direction of the camera
  */
void Camera::setTranslationY(float value) 
{
	translationY = value;
}

/**
  * Sets the zoom of the camera
  * @params value zoom of the camera
  */
void Camera::setZoom(float value) 
{
	zoom = value;
}

/**
  * Sets the zoom and the translation of the camera corresponding to the boundaries of the picture
  * @param boundaris 4 element array with the boundaries of the picture following the order xMin, yMin, xMax, yMax
  */
void Camera::setCameraToBoundaries(const float *boundaries) {
	float aspectRatio = width / height;

	float xSize = boundaries[2] - boundaries[0];
	float ySize = boundaries[3] - boundaries[1];
	
	float zoom1;
	float zoom2;
	
	if (aspectRatio >= 1.0f) {
		zoom1 = (xSize / (2.0f * aspectRatio)) * 1.05f;
		zoom2 = ySize * 0.5f * 1.05f;

		zoom = zoom1 > zoom2 ? zoom1 : zoom2;
	} else {
		zoom1 = xSize * 0.5f * 1.05f;
		zoom2 = (ySize / aspectRatio) * 0.5f * 1.05f;
		
		zoom = zoom1 > zoom2 ? zoom1 : zoom2;
	}

	translationX = (boundaries[2] + boundaries[0]) * 0.5f;
	translationY = (boundaries[3] + boundaries[1]) * 0.5f;
}