#include "DataSet.h"
#include <QMessageBox>
#include "common.h"

#include <float.h>

using namespace log4cxx;

LoggerPtr DataSet::logger(Logger::getLogger("DataSet"));

/**
  * Constructs a new DataSet object
  * @param _root pointer to the root node of the tree
  * @param _nodes array of pointers to the nodes of the tree
  * @param _hierarchyElements number of nodes in the tree
  */
DataSet::DataSet(Node *_root, Node **_nodes, int _hierarchyElements)
{
	relationInsertionIndex = 0;

	root = _root;
	nodes = _nodes;
	hierarchyElements = _hierarchyElements;

	rootedTreeBuilt = false;
	bSplineSet = NULL;

	showRelations = true;

	rootedTreeBoundaries = new float[4];
	radialTreeBoundaries = new float[4];
	balloonTreeBoundaries = new float[4];

	for (int i = 0; i < 4; i++)
	{
		if (i == 0 || i == 1)
		{
			rootedTreeBoundaries[i] = FLT_MAX;
			radialTreeBoundaries[i] = FLT_MAX;
			balloonTreeBoundaries[i] = FLT_MAX;
		} else
		{
			rootedTreeBoundaries[i] = -FLT_MAX;
			radialTreeBoundaries[i] = -FLT_MAX;
			balloonTreeBoundaries[i] = -FLT_MAX;
		}
	}

	rootedTreeDisplayListID = 0;
	radialTreeDisplayListID = 0;
	balloonTreeDisplayListID = 0;
}

/**
  * Destroys the object
  */
DataSet::~DataSet()
{
	delete root;

	/*for (int i = 0; i < hierarchyElements; i++)
	{
		delete nodes[i];
	}*/

	delete [] nodes;

	delete bSplineSet;
	delete [] originNodeIndices;
	delete [] destinationNodeIndices;

	delete [] rootedTreeBoundaries;
	delete [] radialTreeBoundaries;
	delete [] balloonTreeBoundaries;

	if (rootedTreeDisplayListID != 0)
	{
		glDeleteLists(rootedTreeDisplayListID, 1);
	}
	if (radialTreeDisplayListID != 0)
	{
		glDeleteLists(radialTreeDisplayListID, 1);
	}
	if (balloonTreeDisplayListID != 0)
	{
		glDeleteLists(balloonTreeDisplayListID, 1);
	}
}

/**
  * Returns the pointer to the array with the pointers to the nodes of the tree
  * @return pointer to the array with the pointers to the nodes of the tree
  */
Node** DataSet::getNodes()
{
	return nodes;
}

/**
  * Returns the pointer to a specific Node object
  * @param index index of the node
  * @return pointer to the considered Node object
  */
Node* DataSet::getNode(int index)
{
	return nodes[index];
}

/**
  * Initializes the arrays for the relation information
  * @param _relationCount number of relations
  */
void DataSet::initRelationArrays(int _relationCount)
{
	relationCount = _relationCount;
	originNodeIndices = new int[relationCount];
	destinationNodeIndices = new int[relationCount];
}

/**
  * Adds a relation
  * @param index index of the relation starting at 0
  * @param originIndex index of the origin node
  * @param destinationIndex index of the destination node
  */
void DataSet::addRelation(int index, int originIndex, int destinationIndex)
{
	bool relationExists = false;
	if (relationInsertionIndex == 0)
	{	originNodeIndices[relationInsertionIndex] = originIndex;
		destinationNodeIndices[relationInsertionIndex] = destinationIndex;
		relationInsertionIndex++;
	} else
	{
		for (int i = 0; i < relationInsertionIndex; i++)
		{
			if (originIndex == originNodeIndices[i] &&
				destinationIndex == destinationNodeIndices[i])
			{
				relationExists = true;
				break;
			}
		}

		if (relationExists)
		{
			relationCount--;
		} else
		{
			originNodeIndices[relationInsertionIndex] = originIndex;
			destinationNodeIndices[relationInsertionIndex] = destinationIndex;
			relationInsertionIndex++;
		}
	}
}

/**
  * Returns the root of the tree
  * @return pointer to the Node object representing the root of the tree
  */
Node* DataSet::getRoot()
{
	return root;
}

/**
  * Renders the Rooted Tree
  */
void DataSet::renderRooted()
{
	if(rootedTreeBuilt)
	{
		glCallList(rootedTreeDisplayListID);

		if(showRelations)
		{
			bSplineSet->renderRooted();
		}
	}
}

/**
  * Renders the Radial Tree
  */
void DataSet::renderRadial()
{	
	if(radialTreeBuilt)
	{
		glCallList(radialTreeDisplayListID);

		if(showRelations)
		{
			bSplineSet->renderRadial();
		}
	}
}

/**
  * Renders the Balloon Tree
  */
void DataSet::renderBalloon()
{
	if(balloonTreeBuilt)
	{
		glCallList(balloonTreeDisplayListID);

		if(showRelations)
		{
			bSplineSet->renderBalloon();
		}
	}
}

/**
  * Sets the number of layers the tree has and makes a new BSplineSet object
  * @param _hierarchyDepth number of layers the tree has
  */
void DataSet::setHierarchyDepth(int _hierarchyDepth)
{
	hierarchyDepth = _hierarchyDepth;
	bSplineSet = new BSplineSet(hierarchyDepth);
}

/**
  * Sets the maximum fanout of the tree, which is the maximum number of childs of every node
  */
void DataSet::setMaxFanOut()
{
	maxFanOut = root->getMaximumFanOutOfSubtree();
}

/**
  * Initializes the dataset (sets the levels of every node, sets the max. fanout, builds the trees and finds the control polygons)
  */
void DataSet::init()
{
	root->setLevelsForSubTree(0);
	setMaxFanOut();

	unsigned int* nextLevelIndices = new unsigned int[hierarchyDepth];
	for(int i = 0; i < hierarchyDepth; i++) nextLevelIndices[i] = 0;
	getRoot()->computeSiblingIndex(nextLevelIndices);

	nodesPerLevel = getRoot()->getNodesPerLevelOfSubtree(hierarchyDepth);
	getRoot()->computeSiblingsOfSubtree(nodesPerLevel);
	
	getRoot()->preComputeRadialSubTree(45.0f * PI / 180.0f, 1, NULL, true, nodesPerLevel);
	int blah = 0;
	Node *node = getRoot()->findNodeAtMaxDepth(NULL, &blah);
	node->preComputeRootedSubTree(true, node->getSiblingIndex());

	buildRootedTree();
	buildRadialTree();
	getRoot()->preComputeBalloonTree(45.0f * PI / 180.0f, 0);
	buildBalloonTree();
	findControlPolygons();
	setTreeBoundaries();
}

/**
  * Builds the Rooted Tree
  */
void DataSet::buildRootedTree()
{
	if (rootedTreeDisplayListID == 0)
	{
		rootedTreeDisplayListID = glGenLists(1);
	} else
	{
		glDeleteLists(rootedTreeDisplayListID, 1);
	}

	glNewList(rootedTreeDisplayListID, GL_COMPILE);
	root->renderRootedSubTree(0.0f, 0.5f, maxFanOut, hierarchyDepth);
	glEndList();

	rootedTreeBuilt = true;
}

/**
  * Builds the Radial Tree
  */
void DataSet::buildRadialTree()
{
	if (radialTreeDisplayListID == 0)
	{
		radialTreeDisplayListID = glGenLists(1);
	} else
	{
		glDeleteLists(radialTreeDisplayListID, 1);
	}

	glNewList(radialTreeDisplayListID, GL_COMPILE);

	unsigned int* nodesPerLevel = root->getNodesPerLevelOfSubtree(hierarchyDepth);
	root->renderRadialSubTree(0.0f, 0.0f, 45.0f * PI / 180.0f, maxFanOut, hierarchyDepth, nodesPerLevel);
	
	glEndList();

	radialTreeBuilt = true;
}

/**
  * Builds the Balloon Tree
  */
void DataSet::buildBalloonTree()
{	
	if (balloonTreeDisplayListID == 0)
	{
		balloonTreeDisplayListID = glGenLists(1);
	} else
	{
		glDeleteLists(balloonTreeDisplayListID, 1);
	}

	glNewList(balloonTreeDisplayListID, GL_COMPILE);

	root->renderBalloonSubTree(0.0f, 0.0f, 45.0f * PI / 180.0f, 0.0f, maxFanOut, hierarchyDepth);

	glEndList();

	balloonTreeBuilt = true;
}

/**
  * Finds the control polygons for all pairs of nodes which have a relation
  */
void DataSet::findControlPolygons()
{
	int maxPathLength = 2 * hierarchyDepth - 1;
	int *pathIndices = new int[maxPathLength];

	Node **controlPolygon = new Node*[maxPathLength];

	int frontIndex;
	int endIndex;
	int tempIndex;

	int levelDifference = 0;
	bool lcaFound = false;
	int lcaIndex = 0;

	for (int i = 0; i < relationCount; i++)
	{
		frontIndex = 0;
		endIndex = maxPathLength;

		pathIndices[frontIndex] = originNodeIndices[i];
		pathIndices[endIndex] = destinationNodeIndices[i];

		levelDifference = getNode(pathIndices[frontIndex])->getLevel() -
			getNode(pathIndices[endIndex])->getLevel();

		frontIndex++;
		endIndex--;

		if (levelDifference > 0)
		{
			for (int j = 0; j < levelDifference; j++)
			{
				pathIndices[frontIndex] = getNode(pathIndices[frontIndex - 1])->getParent()->getID();
				frontIndex++;
			}
		} else if (levelDifference < 0)
		{
			for (int j = levelDifference; j < 0; j++)
			{
				pathIndices[endIndex] = getNode(pathIndices[endIndex + 1])->getParent()->getID();
				endIndex--;
			}
		}

		while (!lcaFound)
		{
			pathIndices[frontIndex] = getNode(pathIndices[frontIndex - 1])->getParent()->getID();
			tempIndex = getNode(pathIndices[endIndex + 1])->getParent()->getID();

			if (pathIndices[frontIndex] == tempIndex)
			{
				//LCA found
				lcaFound = true;
				endIndex++;

				lcaIndex = frontIndex;

			} else
			{
				pathIndices[endIndex] = tempIndex;
				frontIndex++;
				endIndex--;
			}
		}

		lcaFound = false;

		tempIndex = 0;

		for(int j = 0; j <= frontIndex; j++)
		{
			controlPolygon[j] = getNode(pathIndices[j]);
			tempIndex++;
		}

		for(int j = endIndex; j <= maxPathLength; j++)
		{
			controlPolygon[tempIndex] = getNode(pathIndices[j]);
			tempIndex++;
		}

		bSplineSet->addBSpline(controlPolygon, tempIndex, lcaIndex);
	}

	bSplineSet->sortBSplinesByLength();

	//bSplineSet->test();

	delete [] controlPolygon;
}

/**
  * Sets the bundling strength which modifies the control polygon
  * @param bundlingStrength the bundling strength which modifies the control polygon
  */
void DataSet::setBundlingStrength(float bundlingStrength)
{
	if (bSplineSet)
	{
		bSplineSet->setBundlingStrength(bundlingStrength);
	}
}

/**
  * Sets a boolean value indicating whether the relations (B-Splines) are rendered or not
  * @param visable boolean value indicating whether the relations (B-Splines) are rendered or not
  */
void DataSet::setRelationVisability(bool visable)
{
	showRelations = visable;
}

/**
  * Sets the boundaries of the trees
  */
void DataSet::setTreeBoundaries()
{
	root->setBoundariesOfSubTree(rootedTreeBoundaries, radialTreeBoundaries, balloonTreeBoundaries);

	balloonTreeBoundaries[0] *= 1.4f;
	balloonTreeBoundaries[1] *= 1.4f;
	balloonTreeBoundaries[2] *= 1.4f;
	balloonTreeBoundaries[3] *= 1.4f;
}

/**
  * Returns the pointer to the array with the boundaries of the Rooted Tree
  * @return pointer to the array with the boundaries of the Rooted Tree in the form xMin, yMin, xMax, yMax
  */
float* DataSet::getRootedTreeBoundaries()
{
	return rootedTreeBoundaries;
}

/**
  * Returns the pointer to the array with the boundaries of the Radial Tree
  * @return pointer to the array with the boundaries of the Radial Tree in the form xMin, yMin, xMax, yMax
  */
float* DataSet::getRadialTreeBoundaries()
{
	return radialTreeBoundaries;
}

/**
  * Returns the pointer to the array with the boundaries of the Balloon Tree
  * @return pointer to the array with the boundaries of the Balloon Tree in the form xMin, yMin, xMax, yMax
  */
float* DataSet::getBalloonTreeBoundaries()
{
	return balloonTreeBoundaries;
}

/**
  * Sets a boolean value indicating whether the LCA is removed or not
  * @param remove boolean value indicating whether the LCA is removed or not
  */
void DataSet::setLcaRemoval(bool remove)
{
	bSplineSet->setLcaRemoval(remove);
}

/**
  * Sets a boolean value indicating whether transparency is used or not
  * @param useTransparency boolean value indicating whether transparency is used or not
  */
void DataSet::useTransparency(bool useTransparency)
{
	bSplineSet->useTransparency(useTransparency);
}