#include <QtGui>
#include "MainWindow.h"

/**
  * Constructs a new MainWindow object
  * @param parent the parent widget
  */
MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
	fileManager = new FileManager();

	setupUi(this);
	
	connect(actionAbout, SIGNAL(triggered()), this, SLOT(showAboutDialog()));
	connect(actionOpen, SIGNAL(triggered()), this, SLOT(showFileDialog()));
	connect(rootedButton, SIGNAL(clicked()), glWidget, SLOT(setTreeStyleRooted()));
	connect(rootedButton, SIGNAL(clicked()), this, SLOT(disableShowCircleCheckBox()));
	connect(radialButton, SIGNAL(clicked()), glWidget, SLOT(setTreeStyleRadial()));
	connect(radialButton, SIGNAL(clicked()), this, SLOT(enableShowCircleCheckBox()));
	connect(balloonButton, SIGNAL(clicked()), glWidget, SLOT(setTreeStyleBalloon()));
	connect(balloonButton, SIGNAL(clicked()), this, SLOT(enableShowCircleCheckBox()));

	connect(betaSlider, SIGNAL(valueChanged(int)), glWidget, SLOT(setBundlingStrength(int)));

	connect(showRelationsCheckBox, SIGNAL(stateChanged(int)), glWidget, SLOT(setRelationVisability(int)));
	connect(showNodesCheckBox, SIGNAL(stateChanged(int)), glWidget, SLOT(setNodeVisability(int)));
	connect(showHierarchyLinesCheckBox, SIGNAL(stateChanged(int)), glWidget, SLOT(setLineVisability(int)));
	connect(showHierarchyCirclesCheckBox, SIGNAL(stateChanged(int)), glWidget, SLOT(setCircleVisability(int)));
	connect(removeLcaCheckBox, SIGNAL(stateChanged(int)), glWidget, SLOT(setLcaRemoval(int)));
	connect(transparencyCheckBox, SIGNAL(stateChanged(int)), glWidget, SLOT(useTransparency(int)));

	setWindowTitle("Hierarchical Edge Bundles");
}

/**
  * Destroys the object
  */
MainWindow::~MainWindow()
{
	delete fileManager;
}

/**
  * Shows the file dialog to let the user choose a data file
  */
void MainWindow::showFileDialog()
{
	DataSet *dataSet;
	fileName = QFileDialog::getOpenFileName(this,
		tr("Open Data-File"), "data", tr("Data Files (*.rsf)"));

	if (fileName != NULL)
	{	dataSet = fileManager->loadData(&fileName);
		
		dataSet->setBundlingStrength(static_cast<float>(betaSlider->value()) / static_cast<float>(betaSlider->maximum()));
		dataSet->setRelationVisability(showRelationsCheckBox->checkState() == Qt::Checked);
		dataSet->setLcaRemoval(removeLcaCheckBox->checkState() == Qt::Checked);
		dataSet->useTransparency(transparencyCheckBox->checkState() == Qt::Checked);
		dataSet->getRoot()->setNodeVisability(showNodesCheckBox->checkState() == Qt::Checked);
		dataSet->getRoot()->setLineVisability(showHierarchyLinesCheckBox->checkState() == Qt::Checked);
		dataSet->getRoot()->setCircleVisability(showHierarchyCirclesCheckBox->checkState() == Qt::Checked);

		dataSet->buildRootedTree();
		dataSet->buildRadialTree();
		dataSet->buildBalloonTree();

		glWidget->setDataSet(dataSet);
	}
}

/**
  * Shows the About Dialog with some information about the program
  */
void MainWindow::showAboutDialog()
{
	aboutDialog.about(this, "Hierarchical Edge Bundles", "Implementation of the paper from Danny Holten\n\n"
		"Michael Birsak, Michael Hanzl\n" 
		"Info-Vis LU 2009");
}

/**
  * Enables the showHierarchyCirclesCheckBox
  */
void MainWindow::enableShowCircleCheckBox()
{
	showHierarchyCirclesCheckBox->setEnabled(true);
}

/**
  * Disables the showHierarchyCirclesCheckBox
  */
void MainWindow::disableShowCircleCheckBox()
{
	showHierarchyCirclesCheckBox->setEnabled(false);
}