#include <QtGui>
#include <QtOpenGL>

#include <math.h>
#include "MyGLWidget.h"
#include "common.h"

using namespace log4cxx;

LoggerPtr MyGLWidget::logger(Logger::getLogger("MyGLWidget"));

/**
  * Constructs a new MyGLWidget object
  * @param parent the parent widget
  */
MyGLWidget::MyGLWidget(QWidget *parent) : QGLWidget(parent)
{
	 black = QColor::fromRgb(0, 0, 0);
	 dataSet = NULL;
	 treeStyle = RADIAL;

	 nodeDiameter = 0.1f;

	 camera = new Camera();
}

/**
  * Destroys the object
  */
 MyGLWidget::~MyGLWidget()
 {	delete camera;
    makeCurrent();
 }

/**
  * Returns a hint for the minimum size of the widget
  * @return hint for the minimum size of the widget
  */
 QSize MyGLWidget::minimumSizeHint() const
 {
     return QSize(50, 50);
 }

 /**
  * Returns a hint for the size of the widget
  * @return hint for the size of the widget
  */
 QSize MyGLWidget::sizeHint() const
 {
     return QSize(400, 400);
 }

 /**
   * Initializes GL
   */
 void MyGLWidget::initializeGL()
 {
	 qglClearColor(black);
     glShadeModel(GL_FLAT);
     glDisable(GL_DEPTH_TEST);
	 glEnable(GL_BLEND);
	 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	 glEnable(GL_LINE_SMOOTH);
	 glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
     glEnable(GL_CULL_FACE);
 }

/**
  * Renders the whole scene
  */
void MyGLWidget::paintGL()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	float aspectRatio = static_cast<float>(screenWidth) / static_cast<float>(screenHeight);

	camera->apply(aspectRatio);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	if (dataSet)
	{
		switch(treeStyle) {
			case ROOTED:
				renderRootedTree();
				break;

			case RADIAL:
				renderRadialTree();
				break;

			case BALLOON:
				renderBalloonTree();
				break;
		}
	}
}

/**
  * Is called when the size of the widget changes
  * @param width the new width
  * @param height the new height
  */
void MyGLWidget::resizeGL(int width, int height)
{
	glViewport(0, 0, width, height);
	screenWidth = width;
	screenHeight = height;

	updateGL();
}

/**
  * Is called when a mouse click is on the widget
  * @param event the mouse click event
  */
void MyGLWidget::mousePressEvent(QMouseEvent *event)
{
	camera->processMouseEvent(event->button(), true, event->globalX(), event->globalY(), 0, 0,
		this->width(), this->height());

	updateGL();
}

/**
  * Is called when the mouse wheel is moved while the curser is on the widget
  * @param event the mouse wheel event
  */
void MyGLWidget::wheelEvent(QWheelEvent *event) 
{
	camera->processWheelEvent(event);
	updateGL();
}

/**
  * Is called when the mouse button is released on the widget
  * @param event the mouse event
  */
void MyGLWidget::mouseReleaseEvent(QMouseEvent *event)
{	
	camera->processMouseEvent(event->button(), false, event->globalX(), event->globalY(), 0, 0,
		this->width(), this->height());
	updateGL();
}

/**
  * Is called when the mouse is moved on the widget
  * @param event the mouse event
  */
void MyGLWidget::mouseMoveEvent(QMouseEvent *event)
{	
	camera->processMotionEvent(event->globalX(), event->globalY(), 0, 0, this->width(), this->height());
	updateGL();
}

/**
  * Sets the pointer to the dataset object
  * @param _dataSet pointer to the dataset object
  */
void MyGLWidget::setDataSet(DataSet* _dataSet)
{
	if (dataSet)
	{
		delete dataSet;
	}

	dataSet = _dataSet;

	switch (treeStyle)
	{
		case ROOTED:
			camera->setCameraToBoundaries(dataSet->getRootedTreeBoundaries());
			break;

		case RADIAL:
			camera->setCameraToBoundaries(dataSet->getRadialTreeBoundaries());
			break;

		case BALLOON:
			camera->setCameraToBoundaries(dataSet->getBalloonTreeBoundaries());
			break;
	}

	updateGL();
}

/**
  * Sets the tree style to be rooted
  */
void MyGLWidget::setTreeStyleRooted()
{
	treeStyle = ROOTED;

	if (dataSet)
	{
		camera->setCameraToBoundaries(dataSet->getRootedTreeBoundaries());
		updateGL();
	}
}

/**
  * Sets the tree style to be radial
  */
void MyGLWidget::setTreeStyleRadial()
{
	treeStyle = RADIAL;

	if (dataSet)
	{
		camera->setCameraToBoundaries(dataSet->getRadialTreeBoundaries());
		updateGL();
	}
}

/**
  * Sets the tree style to be balloon
  */
void MyGLWidget::setTreeStyleBalloon()
{
	treeStyle = BALLOON;

	if (dataSet)
	{
		camera->setCameraToBoundaries(dataSet->getBalloonTreeBoundaries());
		updateGL();
	}
}

/**
  * Renders the Rooted Tree
  */
void MyGLWidget::renderRootedTree()
{
	glColor3f(1.0f, 1.0f, 1.0f);
	dataSet->renderRooted();
}

/**
  * Renders the Radial Tree
  */
void MyGLWidget::renderRadialTree()
{
	dataSet->renderRadial();
}

/**
  * Renders the Balloon Tree
  */
void MyGLWidget::renderBalloonTree()
{
	dataSet->renderBalloon();
}

/**
  * Sets the bundling strength
  * @param value the bundling strength
  */
void MyGLWidget::setBundlingStrength(int value)
{
	if (dataSet)
	{
		dataSet->setBundlingStrength(static_cast<float>(value) / 3600.0f);
		updateGL();
	}
}

/**
  * Sets a boolean value indicating whether the relations (B-Splines) are rendered or not
  * @param state boolean value indicating whether the relations (B-Splines) are rendered or not
  */
void MyGLWidget::setRelationVisability(int state)
{
	if (dataSet)
	{
		if (state == Qt::Checked)
		{
			dataSet->setRelationVisability(true);
		} else
		{
			dataSet->setRelationVisability(false);
		}

		updateGL();
	}
}

/**
  * Sets a boolean value indicating whether the nodes are rendered or not
  * @param state boolean value indicating whether the nodes are rendered or not
  */
void MyGLWidget::setNodeVisability(int state)
{
	if (dataSet)
	{
		if (state == Qt::Checked)
		{
			dataSet->getRoot()->setNodeVisability(true);
		} else
		{
			dataSet->getRoot()->setNodeVisability(false);
		}

		dataSet->buildRootedTree();
		dataSet->buildRadialTree();
		dataSet->buildBalloonTree();

		updateGL();
	}
}

/**
  * Sets a boolean value indicating whether the hierarchy lines are rendered or not
  * @param state boolean value indicating whether the hierarchy lines are rendered or not
  */
void MyGLWidget::setLineVisability(int state)
{
	if (dataSet)
	{
		if (state == Qt::Checked)
		{
			dataSet->getRoot()->setLineVisability(true);
		} else
		{
			dataSet->getRoot()->setLineVisability(false);
		}

		dataSet->buildRootedTree();
		dataSet->buildRadialTree();
		dataSet->buildBalloonTree();

		updateGL();
	}
}

/**
  * Sets a boolean value indicating whether the hierarchy circles are rendered or not
  * @param state boolean value indicating whether the hierarchy circles are rendered or not
  */
void MyGLWidget::setCircleVisability(int state)
{
	if (dataSet)
	{
		if (state == Qt::Checked)
		{
			dataSet->getRoot()->setCircleVisability(true);
		} else
		{
			dataSet->getRoot()->setCircleVisability(false);
		}

		dataSet->buildRadialTree();
		dataSet->buildBalloonTree();

		updateGL();
	}
}

/**
  * Sets a boolean value indicating whether the LCA is removed or not
  * @param state boolean value indicating whether the LCA is removed or not
  */
void MyGLWidget::setLcaRemoval(int state)
{	
	if (dataSet)
	{
		if (state == Qt::Checked)
		{
			dataSet->setLcaRemoval(true);
		} else
		{
			dataSet->setLcaRemoval(false);
		}

		updateGL();
	}
}

/**
  * Sets a boolean value indicating whether transparency is used or not
  * @param state boolean value indicating whether transparency is used or not
  */
void MyGLWidget::useTransparency(int state)
{	
	if (dataSet)
	{
		if (state == Qt::Checked)
		{
			dataSet->useTransparency(true);
		} else
		{
			dataSet->useTransparency(false);
		}

		updateGL();
	}
}