#include "Node.h"
#include "common.h"

using namespace log4cxx;

LoggerPtr Node::logger(Logger::getLogger("Node"));

/**
  * Constructs a new Node object
  * @param _id the ID of the node
  */
Node::Node(int _id)
{
	parent = NULL;
	leftSibling = NULL;
	rightSibling = NULL;
	id = _id;
	childVector = new std::vector<Node*>();
	level = 0;
	rootedTreeX = 0.0f;
	rootedTreeY = 0.0f;
	
	radialTreeX = 0.0f;
	radialTreeY = 0.0f;

	balloonTreeX = 0.0f;
	balloonTreeY = 0.0f;

	balloonRadiusOfNode = 0.0f;
	balloonAngleOfNode = 0.0f;

	showNodes = true;
	showLines = true;

	radialLeftLimitAngle = radialRightLimitAngle = 0.0f;
	radialLeftBisectorLimitAngle = radialRightBisectorLimitAngle = 0.0f;
	radialAnglePerChild = 0.0f;
	radialAngle = 0.0f;
	isRightestChild = false;
	isLeftestChild = false;

	preRootedX = preRootedY = preRootedXTmp = 0.0f;
}

/**
  * Destroys the object
  */
Node::~Node()
{
	childVector->clear();
	delete childVector;
}

/**
  * Sets the pointer to the parent node
  * @param _parent pointer to the parent node
  */
void Node::setParent(Node *_parent)
{
	parent = _parent;
}

/**
  * Returns a pointer to the parent node
  * @return pointer to the parent node
  */
Node* Node::getParent()
{
	return parent;
}

/**
  * Adds a pointer to a Node object representing a child of the current node to the childVector
  * @param _child pointer to a Node object representing a child of the current node
  */
void Node::addChild(Node *_child)
{
	childVector->push_back(_child);
}

/**
  * Sets the level for each node in the whole subtree starting with actLevel at the root
  * @param actLevel the level of the current node representing the root of the subtree
  */
void Node::setLevelsForSubTree(int actLevel)
{
	level = actLevel;

	std::vector<Node*>::iterator childIterator;

	for (childIterator = childVector->begin(); childIterator != childVector->end(); childIterator++)
	{
		(*childIterator)->setLevelsForSubTree(actLevel + 1);
	}
}

/**
  * Returns the level of the node
  * @return the level of the node
  */
int Node::getLevel()
{
	return level;
}

/**
  * Returns one node, which has maximal depth
  * @param node Found node (NULL allowed)
  * @param maxDepth Maximal depth of the found node
  * @return Node, which has maximal depth
  */
Node* Node::findNodeAtMaxDepth(Node *node, int* maxDepth)
{
	std::vector<Node*>::iterator it;

	if(*maxDepth < this->level)
	{
		node = this;
		*maxDepth = this->level;
	}

	for(it = childVector->begin(); it != childVector->end(); it++)
	{
		node = (*it)->findNodeAtMaxDepth(node, maxDepth);
	}

	return node;
}

/**
  * Improves the rooted tree. Actually the rooted tree is already precomputed in the preComputeRadialSubTree-function
  * This function must be called from one node at the maximum depth. The tree is traversed from right to left
  * and from bottom to top. This function tries to center the nodes. If the order of the sibling nodes at a hierarchical level gets mixed up, all
  * nodes at that hierarchical level are reseted to the original values.
  * @param up Determines if the last step was one up in the hierarchy
  * @param stopIndex siblingIndex to stop traversing the sibling nodes
  */
void Node::preComputeRootedSubTree(bool up, unsigned int stopIndex) 
{
	// stopIndex reached = all nodes at this hierarchical level are traversed -> go up in the hierarchy
	if((siblingIndex == stopIndex) && (up == false))
	{
		// check for overlappings and handle them
		if((this->leftSibling != NULL) && (this->rightSibling != NULL))
		{		
			unsigned int index = this->siblingIndex;
			unsigned int nextNodeIndex = this->siblingIndex + 1;
			Node *nextNode = this;
			Node *leftNode, *rightNode;
			bool overlap = false;
			
			while(index != nextNodeIndex)
			{
				leftNode = nextNode->getLeftSibling();
				rightNode = nextNode->getRightSibling();
				
				if((leftNode->getPreRootedX() < nextNode->getPreRootedX()) && (leftNode->getPreRootedXTmp() >= nextNode->getPreRootedXTmp())) overlap = true; // lastPreRootedX
				else if((rightNode->getPreRootedX() > nextNode->getPreRootedX()) && (rightNode->getPreRootedXTmp() <= nextNode->getPreRootedXTmp())) overlap = true; // lastPreRootedX				

				if(overlap == true)
				{
					// overlappings detected, do a rollback
					nextNode = this->leftSibling;
					index = this->siblingIndex;

					this->preRootedX = this->preRootedXTmp;

					while(index != nextNode->getSiblingIndex())
					{
						if(nextNode->getChildCount() != 0) nextNode->setPreRootedX(nextNode->getPreRootedXTmp());
						
						nextNode = nextNode->getLeftSibling();
						if(nextNode == NULL) break;
					}

					break;
				}
				
				nextNode = nextNode->getLeftSibling();
				nextNodeIndex = nextNode->getSiblingIndex();
				if(nextNode == NULL) break;
			}
		}
		
		if(parent != NULL) parent->preComputeRootedSubTree(true, parent->getSiblingIndex());
		return;
	}

	// Center the node in order to the positions of the children
	if(childVector->size() != 0)
	{
		std::vector<Node*>::iterator it;
		float minX = static_cast<float>(MAXINT32);
		float maxX = static_cast<float>(-MAXINT32);
		float childPreRootedX;
		
		// find min and max x-coordinate of the children
		for(it = childVector->begin(); it != childVector->end(); it++)
		{
			childPreRootedX = (*it)->getPreRootedX();
			if(childPreRootedX < minX) minX = childPreRootedX;
			if(childPreRootedX > maxX) maxX = childPreRootedX;
		}

		float lastPreRootedX = preRootedX;
		this->preRootedXTmp = lastPreRootedX;
		preRootedX = abs(maxX - minX) / 2.0f + minX;
	}
	else
	{
		this->preRootedXTmp = preRootedX;
	}
	
	// if the node has not got siblings
	if(leftSibling != NULL)
	{
		leftSibling->preComputeRootedSubTree(false, stopIndex);
	}
	else
	{
		if(parent != NULL) parent->preComputeRootedSubTree(true, parent->getSiblingIndex());
	}
}

/**
  * Renders the whole subtree as a Rooted Tree
  * @param rootX the x-coordiante of the current node representing the root of the subtree
  * @param rootY the y-coordiante of the current node representing the root of the subtree
  * @param maxFanOut the maximum number of childs for each node in the whole tree
  * @param hierarchyDepth Number of levels of the tree
  */
void Node::renderRootedSubTree(float rootX, float rootY, int maxFanOut, int hierarchyDepth)
{
	if(level == 0)
	{
		rootedTreeX = rootX = preRootedX;
		rootedTreeY = rootY = preRootedY;
	}
	else
	{
		rootedTreeX = rootX;
		rootedTreeY = rootY;
	}

	float nodeDiameter = NODE_DIAMETER / static_cast<float>(level + 1);

	int childCount = childVector->size();

	std::vector<Node*>::iterator it;
	
	float childX, childY;

	for(it = childVector->begin(); it != childVector->end(); it++)
	{	
		childX = (*it)->getPreRootedX();
		childY = (*it)->getPreRootedY();
		
		if(showLines)
		{		
			glColor3f(0.95f, 0.95f, 0.95f);
			glBegin(GL_LINES);
				glVertex3f(rootX, rootY, -0.1f);
				glVertex3f(childX, childY, -0.1f);
			glEnd();
		}
	
		(*it)->renderRootedSubTree(childX, childY, maxFanOut, hierarchyDepth);
	}

	//The rootnode itself
	if (showNodes)
	{
		glColor3f(0.64f, 0.76f, 0.86f);
		drawNode(rootX, rootY, 0.0f, nodeDiameter * 0.5f);

		glColor3f(0.8f, 0.8f, 0.8f);
		drawCircle(rootX, rootY, 0.0f, nodeDiameter * 0.5f);
	}
}

/**
  * Renders the whole subtree as a Rooted Tree
  * @param rootX the x-coordiante of the current node representing the root of the subtree
  * @param rootY the y-coordiante of the current node representing the root of the subtree
  * @param maxFanOut the maximum number of childs for each node in the whole tree
  * @param hierarchyDepth Number of levels of the tree
  */
/*
void Node::renderRootedSubTree(float rootX, float rootY, int maxFanOut, int hierarchyDepth)
{
	rootedTreeX = rootX;
	rootedTreeY = rootY;

	float nodeDiameter = 0.05f;
	float levelDistance = 0.5f;
	float childX = 0.0f;
	float childY = 0.0f;
	float itX = 0.0f;
	int numChilds = childVector->size();

	if (numChilds > 0)
	{
		float leavesOnBottomLayer = 0.0f;
		float widthOnBottomLayer = 0.0f;

		float childIsParrentOfLeaves = 0.0f;
		float withOfChildLeaves = 0.0f;

		leavesOnBottomLayer = pow(static_cast<float>(maxFanOut), static_cast<float>(hierarchyDepth - level - 1));

		widthOnBottomLayer = leavesOnBottomLayer * nodeDiameter +
			(leavesOnBottomLayer - (leavesOnBottomLayer / static_cast<float>(maxFanOut))) * nodeDiameter +
			((leavesOnBottomLayer / static_cast<float>(maxFanOut)) - 1.0f) * 2.0f * nodeDiameter;

		if (level < hierarchyDepth - 2) {
			childIsParrentOfLeaves = leavesOnBottomLayer / maxFanOut;
			
			withOfChildLeaves = childIsParrentOfLeaves * nodeDiameter +
				(childIsParrentOfLeaves - (childIsParrentOfLeaves / static_cast<float>(maxFanOut))) * nodeDiameter +
				((childIsParrentOfLeaves / static_cast<float>(maxFanOut)) - 1.0f) * 2.0f * nodeDiameter;

			itX = withOfChildLeaves + 2.0f * nodeDiameter;

			childX = rootX - (widthOnBottomLayer * 0.5f) + (withOfChildLeaves * 0.5f) + 
				(maxFanOut - numChilds) * itX * 0.5f;
			
		} else {
			itX = nodeDiameter * 2.0f;
			withOfChildLeaves = static_cast<float>(numChilds) * nodeDiameter + 
				static_cast<float>(numChilds - 1) * nodeDiameter;
			childX = rootX - (withOfChildLeaves * 0.5f) + nodeDiameter * 0.5f;
		}

		childY = rootY - levelDistance;

		//The lines between the subtree-root and the children

		std::vector<Node*>::iterator it;

		for (it = childVector->begin(); it != childVector->end(); it++)
		{	
			if (showLines)
			{
				glColor3f(0.95f, 0.95f, 0.95f);
				glBegin(GL_LINES);
					glVertex3f(rootX, rootY, -0.1f);
					glVertex3f(childX, childY, -0.1f);
				glEnd();
			}

			(*it)->renderRootedSubTree(childX, childY, maxFanOut, hierarchyDepth);
			childX += itX;
		}
	}

	//The rootnode itself
	if (showNodes)
	{
		glColor3f(0.64f, 0.76f, 0.86f);
		drawNode(rootX, rootY, 0.0f, nodeDiameter * 0.5f);

		glColor3f(0.8f, 0.8f, 0.8f);
		drawCircle(rootX, rootY, 0.0f, nodeDiameter * 0.5f);
	}
}
*/

/**
  * Precomputes the radial tree and the rooted tree. The tree is traversed from right to left and from the root node to the nodes at the bottom level.
  * @param rootAngle Root angle of the root node
  * @param stopIndex siblingIndex to step traversing the siblings
  * @param childFound One child of the hierarchy level, or NULL if no one was found until now. Used for getting down in the hierarchy.
  * @param down True if the last iteration was a step down in the hierarchy, false otherwise
  * @param nodesPerLevel Count of Nodes per level of the tree
  */
void Node::preComputeRadialSubTree(float rootAngle, unsigned int stopIndex, Node *childFound, bool down, unsigned int* nodesPerLevel)
{
	float nodeDiameter = NODE_DIAMETER / static_cast<float>(level + 1);
	std::vector<Node*>::iterator it;
	int childCount = childVector->size();
	float childDepth;

	if(childFound == NULL)
	{
		if(childCount > 0) 
		{
			it = childVector->begin();
			childFound = (*it);
		}
	}

	// stopIndex reached = all nodes at this hierarchical level are traversed -> go down in the hierarchy
	if((stopIndex == this->siblingIndex) && (down == false))
	{
		if(childFound != NULL) childFound->preComputeRadialSubTree(rootAngle, childFound->getSiblingIndex(), NULL, true, nodesPerLevel);
		return;
	}

	if(level == 0) // the root node
	{
		this->radialAnglePerChild = 2.0f * PI / static_cast<float>(childCount);
		this->radialAngle = rootAngle;
		this->radialLeftLimitAngle = rootAngle;
		this->radialRightLimitAngle = rootAngle;

		// Sets the radial angles of the children and same values for the rooted tree
		std::vector<Node*>::iterator it2;
		float childAngle = this->radialLeftLimitAngle;
		for(it2 = childVector->begin(); it2 != childVector->end(); it2++)
		{
			if(childCount == 1) (*it2)->setRadialAngle(this->radialAngle);
			else (*it2)->setRadialAngle(childAngle);
			
			// for the rooted tree
			childDepth = static_cast<float>((*it2)->getLevel()) * RADIAL_RADIUS_CONST;
			(*it2)->setPreRootedX((childAngle - 45.0f * PI / 180.0f) * childDepth);	
			(*it2)->setPreRootedY(-childDepth);
			
			childAngle -= this->radialAnglePerChild;		
		}
	}
	else if(level == 1)
	{
		float childRadius = static_cast<float>(level + 1) * RADIAL_RADIUS_CONST;

		this->radialAnglePerChild = 2.0f * PI / static_cast<float>(nodesPerLevel[level + 1]);		
		this->radialLeftLimitAngle = this->radialAngle + (static_cast<float>(childCount) / 2.0f - 0.5f) * this->radialAnglePerChild;
		this->radialRightLimitAngle = this->radialAngle - (static_cast<float>(childCount) / 2.0f - 0.5f) * this->radialAnglePerChild;

		// Sets the radial angles of the children and some values for the rooted tree
		std::vector<Node*>::iterator it2;
		float childAngle = this->radialLeftLimitAngle;
		int i = 0;
		for(it2 = childVector->begin(); it2 != childVector->end(); it2++)
		{
			if(i == 0) (*it2)->setIsLeftestChild(true);
			else if(i == childCount - 1) (*it2)->setIsRightestChild(true);
			
			if(childCount == 1) (*it2)->setRadialAngle(this->radialAngle);
			else (*it2)->setRadialAngle(childAngle);
			
			// For the rooted tree
			childDepth = static_cast<float>((*it2)->getLevel()) * RADIAL_RADIUS_CONST;
			(*it2)->setPreRootedX((childAngle - 45.0f * PI / 180.0f) * childDepth);
			(*it2)->setPreRootedY(-childDepth);

			childAngle -= this->radialAnglePerChild;
			i++;
		}
	}
	else if(level >= 2)
	{
		float childRadius = static_cast<float>(level + 1) * RADIAL_RADIUS_CONST;
		float thisRadius = static_cast<float>(level) * RADIAL_RADIUS_CONST;
		//float arcAngle = 4.0f * asin(thisRadius / childRadius); // 4 * arcsin(R_in / R_out) // wrong formula of the paper
		float h = childRadius - thisRadius;
		float arcAngle = 4.0f * atan(h / (sqrt(childRadius * childRadius - (childRadius - h) * (childRadius - h))));
		float leftTangentLimitAngle = radialAngle + arcAngle / 2.0f;
		float rightTangentLimitAngle = radialAngle - arcAngle / 2.0f;

		float freespaceAngle = asin(RADIAL_FREESPACE_ANGLE / thisRadius); // free space between the groups of children

		if(down == true)
		{
			if(childVector->size() == 1)
			{
				this->radialLeftBisectorLimitAngle = this->radialAngle;
				this->radialRightBisectorLimitAngle = this->radialAngle;
			}
			else
			{
				this->radialLeftBisectorLimitAngle = this->radialAngle + parent->getRadialAnglePerChild() / 2.0f;
				this->radialRightBisectorLimitAngle = this->radialAngle - parent->getRadialAnglePerChild() / 2.0f;
			}
		}

		// Precompute the radialBisectorLimitAngles of the sibling nodes
		if((down == true) && (this->leftSibling != NULL) && (this->rightSibling != NULL))
		{
			Node *nextSibling = this->leftSibling;
			unsigned int nextSiblingIndex = nextSibling->getSiblingIndex();
			while(nextSiblingIndex != this->siblingIndex)
			{
				nextSibling->computeRadialBisectorLimitAngles();
				nextSibling = nextSibling->getLeftSibling();
				nextSiblingIndex = nextSibling->getSiblingIndex();
			}
		}
		
		// Consider the bisector limits of the left sibling nodes and calculate the left bisector limit of that node
		Node* leftNode = this->leftSibling;
		if(leftNode != NULL)
		{
			if(leftNode->getChildCount() == 0)
			{
				float tmpAngle, lastTmpAngle = this->radialLeftBisectorLimitAngle;

				while(true)
				{
					if(leftNode == NULL) 
					{
						tmpAngle = leftTangentLimitAngle;
						break;
					}

					if(siblingIndex == leftNode->getSiblingIndex()) break;

					//leftNode->computeRightRadialBisectorLimitAngle();

					if(leftNode->getChildCount() > 0) break;
					tmpAngle = leftNode->getRadialRightBisectorLimitAngle();
					
					if((tmpAngle < 0.0f) && (leftTangentLimitAngle > 0.0f)) // (tmpAngle > 0.0f && leftTangentLimitAngle < 0.0f) || 
					{
						if(tmpAngle < leftTangentLimitAngle)
						{							
							tmpAngle = lastTmpAngle;
							break;
						}
					}
					else if(tmpAngle > leftTangentLimitAngle)
					{
						//if(leftNode->getRadialLeftBisectorLimitAngle() < leftTangentLimitAngle) tmpAngle = leftNode->getRadialLeftBisectorLimitAngle();
						/*else*/ tmpAngle = lastTmpAngle;
						break;
					}
					
					lastTmpAngle = tmpAngle;
					leftNode = leftNode->getLeftSibling();
				}
				this->radialLeftBisectorLimitAngle = tmpAngle;
			}
		}
		else
		{
			this->radialLeftBisectorLimitAngle = leftTangentLimitAngle;
		}
		
		// Consider the bisector limits of the right sibling nodes and calculate the right bisector limit of that node
		Node* rightNode = this->rightSibling;
		if(rightNode != NULL)
		{
			if(rightNode->getChildCount() == 0)
			{
				float tmpAngle, lastTmpAngle = this->radialRightBisectorLimitAngle;

				while(true)
				{
					if(rightNode == NULL) 
					{
						tmpAngle = rightTangentLimitAngle;
						break;
					}

					if(siblingIndex == rightNode->getSiblingIndex()) break;

					//rightNode->computeLeftRadialBisectorLimitAngle();

					if(rightNode->getChildCount() > 0) break;
					tmpAngle = rightNode->getRadialLeftBisectorLimitAngle();
					
					if((tmpAngle < 0.0f) && (rightTangentLimitAngle > 0.0f)) // (tmpAngle > 0.0f && rightTangentLimitAngle < 0.0f) || 
					{
						if(tmpAngle > rightTangentLimitAngle)
						{	
							tmpAngle = lastTmpAngle;
							break;
						}
					}				
					else if(tmpAngle < rightTangentLimitAngle)
					{	
						/*if(rightNode->getRadialRightBisectorLimitAngle() > rightTangentLimitAngle) tmpAngle = rightNode->getRadialRightBisectorLimitAngle();*/
						/*else*/ tmpAngle = lastTmpAngle;
						break;
					}
					
					lastTmpAngle = tmpAngle;
					rightNode = rightNode->getRightSibling();
				}
				this->radialRightBisectorLimitAngle = tmpAngle;
			}
		}
		else
		{
			this->radialRightBisectorLimitAngle = rightTangentLimitAngle;
		}
		
		float leftBisectorLimitAngle = this->radialLeftBisectorLimitAngle;
		float rightBisectorLimitAngle = this->radialRightBisectorLimitAngle;	

		// If that node has not got any siblings, the computation of the limits of that node is very easy
		if(this->leftSibling == NULL)
		{
			this->radialLeftLimitAngle = leftTangentLimitAngle;
			this->radialRightLimitAngle = rightTangentLimitAngle;

			if((radialLeftLimitAngle < 0.0f) && (radialRightLimitAngle > 0.0f)) this->radialAnglePerChild = (abs(leftTangentLimitAngle - rightTangentLimitAngle) - freespaceAngle * 2.0f) / static_cast<float>((childCount > 2) ? childCount - 1 : childCount);
			this->radialAnglePerChild = (abs(leftTangentLimitAngle - rightTangentLimitAngle) - freespaceAngle * 2.0f) / static_cast<float>((childCount > 2) ? childCount - 1 : childCount);
		}
		else
		{
			if(leftBisectorLimitAngle < leftTangentLimitAngle) this->radialLeftLimitAngle = leftBisectorLimitAngle;
			else this->radialLeftLimitAngle = leftTangentLimitAngle;
			if(rightTangentLimitAngle > rightBisectorLimitAngle) this->radialRightLimitAngle = rightTangentLimitAngle;
			else this->radialRightLimitAngle = rightBisectorLimitAngle;
			
			if((radialLeftLimitAngle < 0.0f) && (radialRightLimitAngle > 0.0f)) this->radialAnglePerChild = (abs(this->radialLeftLimitAngle - this->radialRightLimitAngle) + freespaceAngle * 2.0f) / static_cast<float>((childCount > 2) ? childCount - 1 : childCount);
			this->radialAnglePerChild = (abs(this->radialLeftLimitAngle - this->radialRightLimitAngle) - freespaceAngle * 2.0f) / static_cast<float>((childCount > 2) ? childCount - 1 : childCount);
		}

		// Sets the radial angles of the children and some values for the rooted tree
		std::vector<Node*>::iterator it2;
		float childAngle;
		if((radialLeftLimitAngle < 0.0f) && (radialRightLimitAngle > 0.0f)) childAngle = this->radialLeftLimitAngle + freespaceAngle;
		else childAngle = this->radialLeftLimitAngle - freespaceAngle;
		int i = 0;
		for(it2 = childVector->begin(); it2 != childVector->end(); it2++)
		{
			if(i == 0) (*it2)->setIsLeftestChild(true);
			else if(i == childCount - 1) (*it2)->setIsRightestChild(true);
			
			if(childCount == 1) (*it2)->setRadialAngle(this->radialAngle);
			else (*it2)->setRadialAngle(childAngle);
			
			// For the rooted tree
			childDepth = static_cast<float>((*it2)->getLevel()) * RADIAL_RADIUS_CONST;
			(*it2)->setPreRootedX((childAngle - 45.0f * PI / 180.0f) * childDepth);			
			(*it2)->setPreRootedY(-childDepth);

			childAngle -= this->radialAnglePerChild;
			i++;
		}
	}

	if(this->leftSibling != NULL)
	{
		this->leftSibling->preComputeRadialSubTree(rootAngle, stopIndex, childFound, false, nodesPerLevel);
	}
	else
	{
		if(childFound != NULL) childFound->preComputeRadialSubTree(rootAngle, childFound->getSiblingIndex(), NULL, true, nodesPerLevel);
	}
}

/**
  * Precomputes the left and right bisector limit angles of the childs
  */
void Node::computeRadialBisectorLimitAngles()
{
	if(childVector->size() == 1)
	{
		this->radialLeftBisectorLimitAngle = this->radialAngle;
		this->radialRightBisectorLimitAngle = this->radialAngle;
	}
	else
	{
		this->radialLeftBisectorLimitAngle = this->radialAngle + parent->getRadialAnglePerChild() / 2.0f;
		this->radialRightBisectorLimitAngle = this->radialAngle - parent->getRadialAnglePerChild() / 2.0f;
	}
}

/**
  * Precomputes the left bisector limit angle of the childs
  */
void Node::computeLeftRadialBisectorLimitAngle()
{
	this->radialLeftBisectorLimitAngle = this->radialAngle + parent->getRadialAnglePerChild() / 2.0f;
}

/**
  * Precomputes the right bisector limit angle of the childs
  */
void Node::computeRightRadialBisectorLimitAngle()
{
	this->radialRightBisectorLimitAngle = this->radialAngle - parent->getRadialAnglePerChild() / 2.0f;
}

/**
  * Renders the whole subtree as a Radial Tree
  * @param rootX the x-coordiante of the current node representing the root of the subtree
  * @param rootY the y-coordiante of the current node representing the root of the subtree
  * @param rootAngle Root angle
  * @param maxFanOut the maximum number of childs for each node in the whole tree
  * @param hierarchyDepth Number of levels of the tree
  * @param nodesPerLevel Count of Nodes per level of the tree
  */
void Node::renderRadialSubTree(float rootX, float rootY, float rootAngle, int maxFanOut, int hierarchyDepth, unsigned int* nodesPerLevel) 
{
	radialTreeX = rootX;
	radialTreeY = rootY;

	float nodeDiameter = NODE_DIAMETER / static_cast<float>(level + 1);

	int childCount = childVector->size();

	std::vector<Node*>::iterator it;
	
	int i = 0;
	float childX, childY;
	float childRadius;
	int childLevel;

	childLevel = level + 1;	
	childRadius = static_cast<float>(childLevel) * RADIAL_RADIUS_CONST;

	for(it = childVector->begin(); it != childVector->end(); it++)
	{	
		childLevel = (*it)->getLevel();
		childX = 0.0f + childRadius * cos((*it)->getRadialAngle());
		childY = 0.0f + childRadius * sin((*it)->getRadialAngle());
		
		if(showLines)
		{		
			glColor3f(0.95f, 0.95f, 0.95f);
			glBegin(GL_LINES);
				glVertex3f(rootX, rootY, -0.1f);
				glVertex3f(childX, childY, -0.1f);
			glEnd();
		}
	
		(*it)->renderRadialSubTree(childX, childY, (*it)->getRadialAngle(), maxFanOut, hierarchyDepth, nodesPerLevel);
		i++;
	}
	
	if(showCircles)
	{
		glColor3f(0.52f, 0.53f, 0.76f);
		drawCircle(0.0f, 0.0f, -0.2f, RADIAL_RADIUS_CONST * static_cast<float>(level));
	}
	
	if(showNodes)
	{
		glColor3f(0.64f, 0.76f, 0.86f);
		drawNode(rootX, rootY, 0.0f, nodeDiameter * 0.5f);

		glColor3f(0.8f, 0.8f, 0.8f);
		drawCircle(rootX, rootY, 0.0f, nodeDiameter * 0.5f);
	}
}

/**
  * Renders the whole subtree as a Balloon Tree
  * @param rootX the x-coordiante of the current node representing the root of the subtree
  * @param rootY the y-coordiante of the current node representing the root of the subtree
  * @param rootAngle The angle of the root node
  * @param radius The current radius
  * @param maxFanOut the maximum number of childs for each node in the whole tree
  * @param hierarchyDepth Number of levels of the tree
  * @return The maximum of the radii of all children
  */
float Node::renderBalloonSubTree(float rootX, float rootY, float rootAngle, float radius, int maxFanOut, int hierarchyDepth)
{	
	balloonTreeX = rootX;
	balloonTreeY = rootY;

	float nodeDiameter = NODE_DIAMETER / static_cast<float>(level + 1);

	int childCount = childVector->size();
	float radiusOfChild, maxRadiusOfChildren = 0.0f;
	
	if(childCount > 0) 
	{
		std::vector<Node*>::iterator it;
		
		int i = 0;
		float childX, childY;
		float childAngle;
		float childRadius = 0.0f;

		for(it = childVector->begin(); it != childVector->end(); it++)
		{				
			childAngle = (*it)->getBalloonAngleOfNode();
			childRadius = (*it)->getBalloonRadiusOfNode();
			
			childX = rootX + childRadius * cos(childAngle);
			childY = rootY + childRadius * sin(childAngle);
			
			if(showLines)
			{
				glColor3f(0.95f, 0.95f, 0.95f);
				glBegin(GL_LINES);
					glVertex3f(rootX, rootY, -0.1f);
					glVertex3f(childX, childY, -0.1f);
				glEnd();
			}

			radiusOfChild = (*it)->renderBalloonSubTree(childX, childY, childAngle, radius + childRadius, maxFanOut, hierarchyDepth);
			if(radiusOfChild > maxRadiusOfChildren) maxRadiusOfChildren = radiusOfChild;
			i++;
		}		

		if(showCircles)
		{
			glColor3f(0.52f, 0.53f, 0.76f);
			drawCircle(rootX, rootY, -0.2f, radius - maxRadiusOfChildren);
		}
	}
	else {
		maxRadiusOfChildren = radius + BALLOON_NODE_CIRCLE_OFFSET;
	}
	
	//The node itself
	if (showNodes)
	{	
		glColor3f(0.64f, 0.76f, 0.86f);
		drawNode(rootX, rootY, 0.0f, nodeDiameter * 0.5f);

		glColor3f(0.8f, 0.8f, 0.8f);
		drawCircle(rootX, rootY, 0.0f, nodeDiameter * 0.5f);
	}

	return maxRadiusOfChildren;
}

/**
  * Precomputes the Balloon Tree. The tree is traversed from bottom to top.
  * @param parentAngle Angle of the parent node
  * @param childCountOfParentNode Number of childs of the parent node
  * @return The radius of the node
  */
float Node::preComputeBalloonTree(float parentAngle, unsigned int childCountOfParentNode)
{
	float childRadius = 0.0f;
	unsigned int childCount = childVector->size();
	float anglePerChild;

	balloonAngleOfNode = parentAngle;

	if (childCount == 0)
	{
		balloonRadiusOfNode = BALLOON_RADIUS_CONST;
	} 
	else 
	{
		std::vector<Node*>::iterator it;
		unsigned int maxDepth = getMaxDepthOfSubtree();
		float angle = parentAngle + PI - (2.0f * PI - PI / static_cast<float>(childCount) / 2.0f) / static_cast<float>(childCount) / 2.0f;
		unsigned int childCountWithChildNodes = 0;
		float maxChildRadius = 0.0f;

		for(it = childVector->begin(); it != childVector->end(); it++)
		{	
			if(maxDepth == 1) anglePerChild = (2.0f * PI - PI / static_cast<float>(childCount) / 2.0f) / static_cast<float>(childCount);
			else anglePerChild = 2.0f * PI / static_cast<float>(childCount);

			angle += anglePerChild;
			childRadius = (*it)->preComputeBalloonTree(angle, childCount);
			
			if(childRadius > maxChildRadius) maxChildRadius = childRadius;
		}

		float radiusOfChildNode;

		if(level != 0) 
		{
			radiusOfChildNode = maxChildRadius + BALLOON_RADIUS_CONST * static_cast<float>(maxDepth - 1);

			if((childCountOfParentNode != 1) && (childCountOfParentNode != 2))
			{
				radiusOfChildNode += BALLOON_NODE_CIRCLE_OFFSET;
				balloonRadiusOfNode = radiusOfChildNode / sin(2.0f * PI / static_cast<float>(childCountOfParentNode) / 2.0f);
			}
			else 
			{					
				if((maxDepth == 1) || (childCountOfParentNode == 1)) balloonRadiusOfNode = maxChildRadius + BALLOON_NODE_CIRCLE_OFFSET;
				else balloonRadiusOfNode = 2.0f * maxChildRadius;
			}
		}
	}

	return balloonRadiusOfNode;
}

/**
  * Sets the angle of the node in the balloon tree
  * @param angle Angle of the node
  */
void Node::setAngleOfNode(float angle)
{
	balloonAngleOfNode = angle;
}

/**
  * Sets the radius of the node in the balloon tree
  * @return Radius of the node
  */
float Node::getBalloonRadiusOfNode()
{
	return balloonRadiusOfNode;
}

/**
  * Returns the angle of the node in the balloon tree
  * @return Angle of the node
  */
float Node::getBalloonAngleOfNode()
{
	return balloonAngleOfNode;
}

/**
  *
  * @param x x-coordinate of the midpoint
  * @param y y-coordinate of the midpoint
  * @param z z-coordinate of the midpoint
  * @param radius radius of the circle
  */
void Node::drawCircle(float x, float y, float z, float radius)
{
	const float angleIncrement = 5.0f * PI / 180.0f;

	glBegin(GL_LINE_LOOP);
	for(float angle = 0.0f; angle < 2.0f * PI; angle += angleIncrement)
	{
		glVertex3f(x + cos(angle) * radius, y + sin(angle) * radius, z);
	}
	glEnd();
}

/**
  * Draws a node as a filled circle at the specified position with the specified radius
  * @param x x-coordinate of the node
  * @param y y-coordinate of the node
  * @param z z-coordinate of the node
  * @param radius radius of the circle that is drawn
  */
void Node::drawNode(float x, float y, float z, float radius)
{
	const float angleIncrement = 5.0f * PI / 180.0f;

	glBegin(GL_TRIANGLE_FAN);
	for(float angle = 0.0f; angle <= 2.0f * PI; angle += angleIncrement)
	{
		glVertex3f(x + cos(angle) * radius, y + sin(angle) * radius, z);
	}
	glEnd();
}

/**
  * Returns the maximum fanout of the subtree, which is the maximum number of childs of every node in the tree
  * @return the maximum fanout of the subtree
  */
int Node::getMaximumFanOutOfSubtree()
{
	if (childVector->size() == 0)
	{
		return 0;

	} else {
		std::vector<Node*>::iterator it;
		int max = childVector->size();
		int temp = 0;

		for (it = childVector->begin(); it != childVector->end(); it++)
		{
			temp = (*it)->getMaximumFanOutOfSubtree();

			if (temp > max)
			{
				max = temp;
			}
		}

		return max;
	}
}

/**
  * Returns the ID of the node
  * @return the ID of the node
  */
int Node::getID()
{
	return id;
}

/**
  * Returns the x-coordinate of the node, when the tree is rendered as a Rooted Tree
  * @return x-coordinate of the node, when the tree is rendered as a Rooted Tree
  */
float Node::getRootedTreeX()
{
	return rootedTreeX;
}

/**
  * Returns the y-coordinate of the node, when the tree is rendered as a Rooted Tree
  * @return y-coordinate of the node, when the tree is rendered as a Rooted Tree
  */
float Node::getRootedTreeY()
{
	return rootedTreeY;
}

/**
  * Returns the x-coordinate of the node, when the tree is rendered as a Radial Tree
  * @return x-coordinate of the node, when the tree is rendered as a Radial Tree
  */
float Node::getRadialTreeX()
{
	return radialTreeX;
}

/**
  * Returns the y-coordinate of the node, when the tree is rendered as a Radial Tree
  * @return y-coordinate of the node, when the tree is rendered as a Radial Tree
  */
float Node::getRadialTreeY()
{
	return radialTreeY;
}

/**
  * Returns the x-coordinate of the node, when the tree is rendered as a Balloon Tree
  * @return x-coordinate of the node, when the tree is rendered as a Balloon Tree
  */
float Node::getBalloonTreeX()
{
	return balloonTreeX;
}

/**
  * Returns the y-coordinate of the node, when the tree is rendered as a Balloon Tree
  * @return y-coordinate of the node, when the tree is rendered as a Balloon Tree
  */
float Node::getBalloonTreeY()
{
	return balloonTreeY;
}

/**
  * Determines the number of nodes in that subtree
  * @return The Number of nodes in that subtree
  */
unsigned int Node::getNodeCountOfSubtree()
{
	unsigned int nodeCount = 0;

	std::vector<Node*>::iterator it;
		
	for(it = childVector->begin(); it != childVector->end(); it++)
	{
		nodeCount = (*it)->getNodeCount(nodeCount);
	}

	nodeCount++; // this node

	return nodeCount;
}

/**
  * Call getNodeCountOfSubtree!
  * @param nodeCount current count of nodes
  * @return The count of nodes.
  */
unsigned int Node::getNodeCount(unsigned int nodeCount)
{
	std::vector<Node*>::iterator it;

	for(it = childVector->begin(); it != childVector->end(); it++)
	{
		nodeCount = (*it)->getNodeCount(nodeCount);
	}

	nodeCount++;

	return nodeCount;
}

/**
  * Determines the maximal depth of the subtree
  * @return The maximal depth of the subtree
  */
unsigned int Node::getMaxDepthOfSubtree()
{
	unsigned int maxDepth = 0;
	std::vector<Node*>::iterator it;
	std::vector<unsigned int> depths;

	for (it = childVector->begin(); it != childVector->end(); it++)
	{
		(*it)->getDepths(&depths, 0);
	}

	std::vector<unsigned int>::iterator it2;

	for (it2 = depths.begin(); it2 != depths.end(); it2++)
	{
		if((*it2) > maxDepth) maxDepth = (*it2);
	}

	return maxDepth;
}

/**
  * Determines the minimal depth of the subtree
  * @return The minimal depth of the subtree
  */
unsigned int Node::getMinDepthOfSubtree()
{
	unsigned int minDepth = MAXINT32;
	
	if(childVector->size() == 0) 
	{
		minDepth = 0;
	}
	else
	{
		std::vector<Node*>::iterator it;
		std::vector<unsigned int> depths;
		
		for (it = childVector->begin(); it != childVector->end(); it++)
		{
			(*it)->getDepths(&depths, 0);
		}

		std::vector<unsigned int>::iterator it2;

		for (it2 = depths.begin(); it2 != depths.end(); it2++)
		{
			if((*it2) < minDepth) minDepth = (*it2);
		}
	}

	return minDepth;
}

/**
  * Returns all depths of the subtree
  * @param depths Computed depths
  * @param currentDepth Current depth
  */
void Node::getDepths(std::vector<unsigned int>* depths, unsigned int currentDepth)
{
	currentDepth++;
	std::vector<Node*>::iterator it;

	if(childVector->size() == 0) 
	{
		depths->push_back(currentDepth);
	}

	for (it = childVector->begin(); it != childVector->end(); it++)
	{
		(*it)->getDepths(depths, currentDepth);
	}
}

/**
  * Returns the maximum of the number of nodes at all levels.
  * @param hierarchyDepth The maximum hierarchical depth of the tree
  * @return The maximum of the number of nodes at all levels
  */
unsigned int Node::getMaxNodesPerLevelOfSubtree(int hierarchyDepth)
{
	unsigned int* nodesPerLevel = getNodesPerLevelOfSubtree(hierarchyDepth);
	unsigned int max = 0;
	for(int i = 0; i < hierarchyDepth; i++)
	{
		if(nodesPerLevel[i] > max) max = nodesPerLevel[i];
	}

	return max;
}

/**
  * Determines the nodes per hierarchical level of the subtree
  * @param hierarchyDepth Maximum depth of the tree
  * @return Nodes per hierarchical level of the subtree
  */
unsigned int* Node::getNodesPerLevelOfSubtree(int hierarchyDepth)
{
	unsigned int* nodesPerLevel = new unsigned int[hierarchyDepth];

	for(int i = 0; i < hierarchyDepth; i++) 
	{
		nodesPerLevel[i] = 0;
	}

	getNodesPerLevel(nodesPerLevel);

	return nodesPerLevel;
}

/**
  * Call getNodesPerLevelOfSubtree!
  * @param nodesPerLevel Current data structure
  */
void Node::getNodesPerLevel(unsigned int* nodesPerLevel)
{
	nodesPerLevel[level]++;
	if(childVector->size() != 0)
	{
		std::vector<Node*>::iterator it;
		for (it = childVector->begin(); it != childVector->end(); it++)
		{
			(*it)->getNodesPerLevel(nodesPerLevel);
		}
	} 
}

/**
  * Returns the number of children of the node
  * @return the number of children of the node
  */
int Node::getChildCount() 
{
	return childVector->size();
}

/**
  * Sets a boolean value indicating whether the nodes should be rendered or not
  * @param visable a boolean value indicating whether the nodes should be rendered or not
  */
void Node::setNodeVisability(bool visable)
{
	showNodes = visable;

	if(childVector->size() != 0)
	{
		std::vector<Node*>::iterator it;
		for (it = childVector->begin(); it != childVector->end(); it++)
		{
			(*it)->setNodeVisability(visable);
		}
	} 
}

/**
  * Sets a boolean value indicating whether the hierarchy lines should be rendered or not
  * @param visable a boolean value indicating whether the hierarchy lines should be rendered or not
  */
void Node::setLineVisability(bool visable)
{
	showLines = visable;

	if(childVector->size() != 0)
	{
		std::vector<Node*>::iterator it;
		for (it = childVector->begin(); it != childVector->end(); it++)
		{
			(*it)->setLineVisability(visable);
		}
	} 
}

/**
  * Sets a boolean value indicating whether the hierarchy circles should be rendered or not
  * @param visable a boolean value indicating whether the hierarchy circles should be rendered or not
  */
void Node::setCircleVisability(bool visable)
{
	showCircles = visable;

	if(childVector->size() != 0)
	{
		std::vector<Node*>::iterator it;
		for (it = childVector->begin(); it != childVector->end(); it++)
		{
			(*it)->setCircleVisability(visable);
		}
	} 
}

/**
  * Sets the boundaries of the subtree
  * @param rootedBoundaries pointer to an array to store the boundaries of the Rooted Tree in the form xMin, yMin, xMax, yMax
  * @param radialBoundaries pointer to an array to store the boundaries of the Radial Tree in the form xMin, yMin, xMax, yMax
  * @param balloonBoundaries pointer to an array to store the boundaries of the Balloon Tree in the form xMin, yMin, xMax, yMax
  */
void Node::setBoundariesOfSubTree(float *rootedBoundaries, float *radialBoundaries, float *balloonBoundaries)
{	
	//Rooted Tree Boundaries
	if (rootedTreeX < rootedBoundaries[0])
	{
		rootedBoundaries[0] = rootedTreeX;
	}
	if (rootedTreeX > rootedBoundaries[2])
	{
		rootedBoundaries[2] = rootedTreeX;
	}
	if (rootedTreeY < rootedBoundaries[1])
	{
		rootedBoundaries[1] = rootedTreeY;
	}
	if (rootedTreeY > rootedBoundaries[3])
	{
		rootedBoundaries[3] = rootedTreeY;
	}

	//Radial Tree Boundaries
	if (radialTreeX < radialBoundaries[0])
	{
		radialBoundaries[0] = radialTreeX;
	}
	if (radialTreeX > radialBoundaries[2])
	{
		radialBoundaries[2] = radialTreeX;
	}
	if (radialTreeY < radialBoundaries[1])
	{
		radialBoundaries[1] = radialTreeY;
	}
	if (radialTreeY > radialBoundaries[3])
	{
		radialBoundaries[3] = radialTreeY;
	}

	//Balloon Tree Boundaries
	if (balloonTreeX < balloonBoundaries[0])
	{
		balloonBoundaries[0] = balloonTreeX;
	}
	if (balloonTreeX > balloonBoundaries[2])
	{
		balloonBoundaries[2] = balloonTreeX;
	}
	if (balloonTreeY < balloonBoundaries[1])
	{
		balloonBoundaries[1] = balloonTreeY;
	}
	if (balloonTreeY > balloonBoundaries[3])
	{
		balloonBoundaries[3] = balloonTreeY;
	}

	if(childVector->size() != 0)
	{
		std::vector<Node*>::iterator it;
		for (it = childVector->begin(); it != childVector->end(); it++)
		{
			(*it)->setBoundariesOfSubTree(rootedBoundaries, radialBoundaries, balloonBoundaries);
		}
	} 
}

/**
  * Computes the siblings of the subtree.
  * @param nodesPerLevel Count of nodes per hierarchical level of the tree
  */
void Node::computeSiblingsOfSubtree(unsigned int* nodesPerLevel)
{
	std::vector<Node*>::iterator it;
	
	leftSibling = getLeftSiblingOfNode(nodesPerLevel);
	rightSibling = getRightSiblingOfNode(nodesPerLevel);
	
	/*
	LOG4CXX_DEBUG(logger, ("level: " + toString(level) + " nodesPerLevel: " + toString(nodesPerLevel[level]) + " siblingIndex: " + toString(siblingIndex)).c_str());
	if(leftSibling != NULL) LOG4CXX_DEBUG(logger, ("leftSibling - Level: " + toString(leftSibling->getLevel()) + " siblingIndex: " + toString(leftSibling->getSiblingIndex())).c_str());
	if(rightSibling != NULL) LOG4CXX_DEBUG(logger, ("rightSibling - Level: " + toString(rightSibling->getLevel()) + " siblingIndex: " + toString(rightSibling->getSiblingIndex())).c_str());
	*/

	for(it = childVector->begin(); it != childVector->end(); it++)
	{
		(*it)->computeSiblingsOfSubtree(nodesPerLevel);
	}
}

/**
  * Determines the left sibling of the current node
  * @param nodesPerLevel Count of nodes per hierarchical level of the tree
  * @return The left sibling of that node
  */
Node* Node::getLeftSiblingOfNode(unsigned int* nodesPerLevel)
{
	if(parent == NULL) return NULL;
	if(nodesPerLevel[level] == 1) return NULL;
	return parent->getLeftSibling(this, this, true);
}

/**
  * Determines the right sibling of the current node
  * @param nodesPerLevel Count of nodes per hierarchical level of the tree
  * @return The right sibling of that node
  */
Node* Node::getRightSiblingOfNode(unsigned int* nodesPerLevel)
{
	if(parent == NULL) return NULL;
	if(nodesPerLevel[level] == 1) return NULL;
	return parent->getRightSibling(this, this, true);
}

/**
  * Call getLeftSiblingOfNode!
  * @param node Node, which left sibling should be determined
  * @param lastNode Last node which was traversed
  * @param up True if the last step was one up in the hierarchy, false otherwise
  * @return Node which should be processed
  */
Node* Node::getLeftSibling(Node* node, Node* lastNode, bool up)
{
	// previous node is at level 1 and here we are at level 0 - so we have to go one step up, get the left sibling and go one step down
	if((level == 0) && (node->getLevel() == 1))
	{
		std::vector<Node*>::iterator it;
		std::vector<Node*>::iterator it2;
		Node *prevNode = NULL;
		Node *leftChild = NULL;

		for(it = childVector->begin(); it != childVector->end(); it++)
		{
			if((*it) == node) 
			{
				leftChild = prevNode;
				if(leftChild == NULL)
				{
					it2 = childVector->end();
					it2--;
					leftChild = (*it2);
				}
				return leftChild;
			}
			prevNode = (*it);
		}
	}
	else if(level == 0) // root
	{
		std::vector<Node*>::iterator it;
		Node *prevNode = NULL;
		Node *leftChild = NULL;
		Node *foundNode = NULL;
		unsigned int startIndex;

		for(it = childVector->begin(); it != childVector->end(); it++)
		{
			if((*it) == lastNode) 
			{
				leftChild = prevNode;
				if(leftChild == NULL)
				{
					it = childVector->end();
					it--;
					leftChild = (*it);
				}
				startIndex = leftChild->getSiblingIndex();
				while(true) 
				{
					foundNode = leftChild->getLeftSibling(node, this, false);
					if(foundNode != NULL) return foundNode;
					it++;
					if(it == childVector->end()) it = childVector->begin();
					leftChild = (*it);
					if(startIndex == leftChild->getSiblingIndex()) return NULL;
				}

				break;
			}
			prevNode = (*it);
		}
	}
	else
	{	
		if(node->getLevel() == level) return this;
		if((up == true) && (childVector->size() == 1)) parent->getLeftSibling(node, this, true);

		std::vector<Node*>::iterator it;
		Node *foundNode = NULL;
		Node *leftChild = NULL;
		Node *prevNode = NULL;
		
		if(up == false)
		{	
			if(childVector->size() == 0) return NULL;

			for(it = childVector->end(); ;)
			{
				it--;
				foundNode = (*it)->getLeftSibling(node, this, false);
				if(foundNode != NULL) return foundNode;
				if(it == childVector->begin()) return NULL;
			}
		}
		else
		{
			for(it = childVector->begin(); it != childVector->end(); it++)
			{
				if((*it) == lastNode) 
				{
					leftChild = prevNode;
					if(leftChild == NULL)
					{
						return parent->getLeftSibling(node, this, true);
					}
					else
					{
						while(true)
						{
							foundNode = leftChild->getLeftSibling(node, this, false);
							if(foundNode != NULL) return foundNode;
							if(it == childVector->begin()) return parent->getLeftSibling(node, this, true);
							it--;
							leftChild = (*it);
						}
					}
					break;
				}
				prevNode = (*it);
			}
		}
	}

	return NULL;
}

/**
  * Call getRightSiblingOfNode!
  * @param node Node, which right sibling should be determined
  * @param lastNode Last node which was traversed
  * @param up True if the last step was one up in the hierarchy, false otherwise
  * @return Node which should be processed
  */
Node* Node::getRightSibling(Node* node, Node* lastNode, bool up)
{
	// previous node is at level 1 and here we are at level 0 - so we have to go one step up, get the right sibling and go one step down
	if((level == 0) && (node->getLevel() == 1))
	{
		std::vector<Node*>::iterator it;

		for(it = childVector->begin(); it != childVector->end(); it++)
		{
			if((*it) == node) 
			{	
				if((it + 1) == childVector->end()) return (*childVector->begin());
				it++;
				return (*it);
			}
		}
	}
	else if(level == 0) // root node
	{
		std::vector<Node*>::iterator it;
		Node *foundNode = NULL;
		Node *rightChild = NULL;
		unsigned int startIndex = 0;

		for(it = childVector->begin(); it != childVector->end(); it++)
		{
			if((*it) == lastNode) 
			{	
				if((it + 1) == childVector->end()) 
				{	
					it = childVector->begin();
					rightChild = (*it);
				}
				else
				{
					it++;
					rightChild = (*it);
				}
				
				startIndex = rightChild->getSiblingIndex();
				while(true) {
					foundNode = rightChild->getRightSibling(node, this, false);
					if(foundNode != NULL) return foundNode;
					it++;
					if(it == childVector->end()) it = childVector->begin();
					rightChild = (*it);
					if(startIndex == rightChild->getSiblingIndex()) return NULL;
				}
				
				break;
			}
		}				
	}
	else
	{
		if(node->getLevel() == level) return this;
		if((up == true) && (childVector->size() == 1)) parent->getRightSibling(node, this, true);
		
		std::vector<Node*>::iterator it;
		Node *foundNode = NULL;
		
		if(up == false)
		{			
			for(it = childVector->begin(); it != childVector->end(); it++)
			{
				foundNode = (*it)->getRightSibling(node, this, false);
				if(foundNode != NULL) return foundNode;
			}
		}
		else
		{
			for(it = childVector->begin(); it != childVector->end(); it++)
			{
				if((*it) == lastNode) 
				{
					for(it++; ;it++)
					{	
						if(it == childVector->end()) return parent->getRightSibling(node, this, true);
						foundNode = (*it)->getRightSibling(node, this, false);
						if(foundNode != NULL) return foundNode;		
					}

					break;
				}
			}
		}
	}

	return NULL;
}

/**
  * Returns the left sibling of that node
  * @return The left sibling of that node
  */
Node* Node::getLeftSibling()
{
	return leftSibling;
}

/**
  * Returns the right sibling of that node
  * @return The right sibling of that node
  */
Node* Node::getRightSibling()
{
	return rightSibling;
}

/**
  * Determines the index of the nodes in one hierarchical level
  * @param nextLevelIndices current indices
  */
void Node::computeSiblingIndex(unsigned int* nextLevelIndices)
{
	siblingIndex = nextLevelIndices[level];
	nextLevelIndices[level]++;

	//LOG4CXX_DEBUG(logger, ("level: " + toString(level) + " index: " + toString(siblingIndex)).c_str());

	if(childVector->size() != 0)
	{
		std::vector<Node*>::iterator it;
		for(it = childVector->begin(); it != childVector->end(); it++)
		{
			(*it)->computeSiblingIndex(nextLevelIndices);		
		}
	}
}

/**
 * Returns the index of the nodes in one hierarchical level
 * @return Index of the nodes in one hierarchical level
 */
unsigned int Node::getSiblingIndex()
{
	return siblingIndex;
}

/**
  * Returns the left limit angle of the childs
  * @return The left limit angle of the childs
  */
float Node::getRadialLeftLimitAngle()
{
	return this->radialLeftLimitAngle;
}

/**
  * Returns the right limit angle of the childs
  * @return The right limit angle of the childs
  */
float Node::getRadialRightLimitAngle()
{
	return this->radialRightLimitAngle;
}

/**
  * Returns the left bisector limit angle of the childs
  * @return The left bisector limit angle of the childs
  */
float Node::getRadialLeftBisectorLimitAngle()
{
	return this->radialLeftBisectorLimitAngle;
}

/**
  * Returns the right bisector limit angle of the childs
  * @return The right bisector limit angle of the childs
  */
float Node::getRadialRightBisectorLimitAngle()
{
	return this->radialRightBisectorLimitAngle;
}

/**
  * Returns the angle for each child of that node
  * @return The angle for each child of that node
  */
float Node::getRadialAnglePerChild()
{
	return this->radialAnglePerChild;
}

/**
  * Returns the angle of that node in the radial tree
  * @return The angle of that node in the radial tree
  */
float Node::getRadialAngle()
{
	return this->radialAngle;
}

/**
  * Sets the left bisector limit angle of the childs
  * @param angle Left bisector limit angle of the childs
  */
void Node::setRadialLeftBisectorLimitAngle(float angle)
{
	this->radialLeftBisectorLimitAngle = angle;
}

/**
  * Sets the right bisector limit angle of the childs
  * @param angle right bisector limit angle of the childs
  */
void Node::setRadialRightBisectorLimitAngle(float angle)
{
	this->radialRightBisectorLimitAngle = angle;
}

/**
  * Determines if the child is the leftest one
  * @return True if the child is the leftest one, false otherwise
  */
bool Node::getIsLeftestChild() 
{
	return isLeftestChild;
}

/**
  * Determines if the child is the rightest one
  * @return True if the child is the rightest one, false otherwise
  */
bool Node::getIsRightestChild()
{
	return isRightestChild;
}

/**
  * Sets if the child is the leftest one
  * @param value True if the child is the leftest one, false otherwise
  */
void Node::setIsLeftestChild(bool value)
{
	isLeftestChild = value;
}

/**
  * Sets if the child is the rightest one
  * @param value True if the child is the rightest one, false otherwise
  */
void Node::setIsRightestChild(bool value)
{
	isRightestChild = value;
}

/**
  * Sets the angle of that node in the radial tree
  * @param angle The angle of that node in the radial tree
  */
void Node::setRadialAngle(float angle)
{
	this->radialAngle = angle;
}

/**
  * Returns the precomputed X-Value of the node when the tree is rendered as Rooted Tree. It is used for transactions.
  * @return The precomputed X-Value of the node when the tree is rendered as Rooted Tree. It is used for transactions.
  */
float Node::getPreRootedXTmp()
{
	return preRootedXTmp;
}

/**
  * Sets the precomputed X-Value of the node when the tree is rendered as Rooted Tree
  * @param value Precomputed X-Value of the node when the tree is rendered as Rooted Tree
  */
void Node::setPreRootedX(float value)
{
	preRootedX = value;
}

/**
  * Sets the precomputed Y-Value of the node when the tree is rendered as Rooted Tree
  * @param value Precomputed Y-Value of the node when the tree is rendered as Rooted Tree
  */
void Node::setPreRootedY(float value)
{
	preRootedY = value;
}

/**
  * Returns the precomputed X-Value of the node when the tree is rendered as Rooted Tree
  * @return The Precomputed X-Value of the node when the tree is rendered as Rooted Tree
  */
float Node::getPreRootedX()
{
	return preRootedX;
}

/**
  * Returns the precomputed Y-Value of the node when the tree is rendered as Rooted Tree
  * @return Precomputed Y-Value of the node when the tree is rendered as Rooted Tree
  */
float Node::getPreRootedY()
{
	return preRootedY;
}